@@ -7,39 +7,37 @@
 use quick_error::quick_error;
 use std::cmp::Ordering;
 use std::collections::{HashMap, HashSet};
-#[cfg(not(windows))]
-use std::ffi::CString;
 use std::ffi::OsString;
-use std::fs::{self, File, Metadata, OpenOptions, Permissions};
+use std::fs::{self, Metadata, OpenOptions, Permissions};
 use std::io;
 #[cfg(unix)]
-use std::os::unix::ffi::OsStrExt;
-#[cfg(unix)]
 use std::os::unix::fs::{FileTypeExt, PermissionsExt};
 use std::path::{Path, PathBuf, StripPrefixError};
+#[cfg(all(unix, not(target_os = "android")))]
+use uucore::fsxattr::copy_xattrs;
 
-use clap::{builder::ValueParser, crate_version, Arg, ArgAction, ArgMatches, Command};
+use clap::{Arg, ArgAction, ArgMatches, Command, builder::ValueParser, value_parser};
 use filetime::FileTime;
 use indicatif::{ProgressBar, ProgressStyle};
-#[cfg(unix)]
-use libc::mkfifo;
 use quick_error::ResultExt;
 
 use platform::copy_on_write;
 use uucore::display::Quotable;
-use uucore::error::{set_exit_code, UClapError, UError, UResult, UUsageError};
+use uucore::error::{UClapError, UError, UResult, UUsageError, set_exit_code};
+#[cfg(unix)]
+use uucore::fs::make_fifo;
 use uucore::fs::{
-    are_hardlinks_to_same_file, canonicalize, get_filename, is_symlink_loop, normalize_path,
-    path_ends_with_terminator, paths_refer_to_same_file, FileInformation, MissingHandling,
-    ResolveMode,
+    FileInformation, MissingHandling, ResolveMode, are_hardlinks_to_same_file, canonicalize,
+    get_filename, is_symlink_loop, normalize_path, path_ends_with_terminator,
+    paths_refer_to_same_file,
 };
 use uucore::{backup_control, update_control};
 // These are exposed for projects (e.g. nushell) that want to create an `Options` value, which
 // requires these enum.
 pub use uucore::{backup_control::BackupMode, update_control::UpdateMode};
 use uucore::{
-    format_usage, help_about, help_section, help_usage, prompt_yes,
-    shortcut_value_parser::ShortcutValueParser, show_error, show_warning,
+    format_usage, help_about, help_section, help_usage,
+    parser::shortcut_value_parser::ShortcutValueParser, prompt_yes, show_error, show_warning,
 };
 
 use crate::copydir::copy_directory;
@@ -51,11 +49,11 @@ quick_error! {
     #[derive(Debug)]
     pub enum Error {
         /// Simple io::Error wrapper
-        IoErr(err: io::Error) { from() source(err) display("{}", err)}
+        IoErr(err: io::Error) { from() source(err) display("{err}")}
 
         /// Wrapper for io::Error with path context
         IoErrContext(err: io::Error, path: String) {
-            display("{}: {}", path, err)
+            display("{path}: {err}")
             context(path: &'a str, err: io::Error) -> (err, path.to_owned())
             context(context: String, err: io::Error) -> (err, context)
             source(err)
@@ -63,7 +61,7 @@ quick_error! {
 
         /// General copy error
         Error(err: String) {
-            display("{}", err)
+            display("{err}")
             from(err: String) -> (err)
             from(err: &'static str) -> (err.to_string())
         }
@@ -73,7 +71,7 @@ quick_error! {
         NotAllFilesCopied {}
 
         /// Simple walkdir::Error wrapper
-        WalkDirErr(err: walkdir::Error) { from() display("{}", err) source(err) }
+        WalkDirErr(err: walkdir::Error) { from() display("{err}") source(err) }
 
         /// Simple std::path::StripPrefixError wrapper
         StripPrefixError(err: StripPrefixError) { from() }
@@ -85,15 +83,15 @@ quick_error! {
         Skipped(exit_with_error:bool) { }
 
         /// Result of a skipped file
-        InvalidArgument(description: String) { display("{}", description) }
+        InvalidArgument(description: String) { display("{description}") }
 
         /// All standard options are included as an an implementation
         /// path, but those that are not implemented yet should return
         /// a NotImplemented error.
-        NotImplemented(opt: String) { display("Option '{}' not yet implemented.", opt) }
+        NotImplemented(opt: String) { display("Option '{opt}' not yet implemented.") }
 
         /// Invalid arguments to backup
-        Backup(description: String) { display("{}\nTry '{} --help' for more information.", description, uucore::execution_phrase()) }
+        Backup(description: String) { display("{description}\nTry '{} --help' for more information.", uucore::execution_phrase()) }
 
         NotADirectory(path: PathBuf) { display("'{}' is not a directory", path.display()) }
     }
@@ -108,15 +106,16 @@ impl UError for Error {
 pub type CopyResult<T> = Result<T, Error>;
 
 /// Specifies how to overwrite files.
-#[derive(Clone, Copy, Eq, PartialEq)]
+#[derive(Debug, Clone, Copy, Eq, PartialEq, Default)]
 pub enum ClobberMode {
     Force,
     RemoveDestination,
+    #[default]
     Standard,
 }
 
 /// Specifies whether files should be overwritten.
-#[derive(Clone, Copy, Eq, PartialEq)]
+#[derive(Debug, Clone, Copy, Eq, PartialEq)]
 pub enum OverwriteMode {
     /// [Default] Always overwrite existing files
     Clobber(ClobberMode),
@@ -126,18 +125,39 @@ pub enum OverwriteMode {
     NoClobber,
 }
 
+impl Default for OverwriteMode {
+    fn default() -> Self {
+        Self::Clobber(ClobberMode::default())
+    }
+}
+
 /// Possible arguments for `--reflink`.
-#[derive(Copy, Clone, Eq, PartialEq)]
+#[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub enum ReflinkMode {
     Always,
     Auto,
     Never,
 }
 
+impl Default for ReflinkMode {
+    #[allow(clippy::derivable_impls)]
+    fn default() -> Self {
+        #[cfg(any(target_os = "linux", target_os = "android", target_os = "macos"))]
+        {
+            ReflinkMode::Auto
+        }
+        #[cfg(not(any(target_os = "linux", target_os = "android", target_os = "macos")))]
+        {
+            ReflinkMode::Never
+        }
+    }
+}
+
 /// Possible arguments for `--sparse`.
-#[derive(Copy, Clone, Eq, PartialEq)]
+#[derive(Debug, Copy, Clone, Eq, PartialEq, Default)]
 pub enum SparseMode {
     Always,
+    #[default]
     Auto,
     Never,
 }
@@ -150,10 +170,11 @@ pub enum TargetType {
 }
 
 /// Copy action to perform
-#[derive(PartialEq)]
+#[derive(Debug, Clone, Eq, PartialEq, Default)]
 pub enum CopyMode {
     Link,
     SymLink,
+    #[default]
     Copy,
     Update,
     AttrOnly,
@@ -172,7 +193,7 @@ pub enum CopyMode {
 /// For full compatibility with GNU, these options should also combine. We
 /// currently only do a best effort imitation of that behavior, because it is
 /// difficult to achieve in clap, especially with `--no-preserve`.
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, Eq, Clone, Copy)]
 pub struct Attributes {
     #[cfg(unix)]
     pub ownership: Preserve,
@@ -183,6 +204,12 @@ pub struct Attributes {
     pub xattr: Preserve,
 }
 
+impl Default for Attributes {
+    fn default() -> Self {
+        Self::NONE
+    }
+}
+
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub enum Preserve {
     // explicit means whether the --no-preserve flag is used or not to distinguish out the default value.
@@ -222,6 +249,7 @@ impl Ord for Preserve {
 ///
 /// The fields are documented with the arguments that determine their value.
 #[allow(dead_code)]
+#[derive(Debug, Clone, Eq, PartialEq)]
 pub struct Options {
     /// `--attributes-only`
     pub attributes_only: bool,
@@ -283,6 +311,47 @@ pub struct Options {
     pub verbose: bool,
     /// `-g`, `--progress`
     pub progress_bar: bool,
+    /// -Z
+    pub set_selinux_context: bool,
+    // --context
+    pub context: Option<String>,
+}
+
+impl Default for Options {
+    fn default() -> Self {
+        Self {
+            attributes_only: false,
+            backup: BackupMode::default(),
+            copy_contents: false,
+            cli_dereference: false,
+            copy_mode: CopyMode::default(),
+            dereference: false,
+            no_target_dir: false,
+            one_file_system: false,
+            overwrite: OverwriteMode::default(),
+            parents: false,
+            sparse_mode: SparseMode::default(),
+            strip_trailing_slashes: false,
+            reflink_mode: ReflinkMode::default(),
+            attributes: Attributes::default(),
+            recursive: false,
+            backup_suffix: backup_control::DEFAULT_BACKUP_SUFFIX.to_owned(),
+            target_dir: None,
+            update: UpdateMode::default(),
+            debug: false,
+            verbose: false,
+            progress_bar: false,
+            set_selinux_context: false,
+            context: None,
+        }
+    }
+}
+
+/// Enum representing if a file has been skipped.
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
+enum PerformedAction {
+    Copied,
+    Skipped,
 }
 
 /// Enum representing various debug states of the offload and reflink actions.
@@ -385,6 +454,7 @@ mod options {
     pub const RECURSIVE: &str = "recursive";
     pub const REFLINK: &str = "reflink";
     pub const REMOVE_DESTINATION: &str = "remove-destination";
+    pub const SELINUX: &str = "Z";
     pub const SPARSE: &str = "sparse";
     pub const STRIP_TRAILING_SLASHES: &str = "strip-trailing-slashes";
     pub const SYMBOLIC_LINK: &str = "symbolic-link";
@@ -413,6 +483,7 @@ const PRESERVE_DEFAULT_VALUES: &str = if cfg!(unix) {
 } else {
     "mode,timestamp"
 };
+
 pub fn uu_app() -> Command {
     const MODE_ARGS: &[&str] = &[
         options::LINK,
@@ -422,7 +493,7 @@ pub fn uu_app() -> Command {
         options::COPY_CONTENTS,
     ];
     Command::new(uucore::util_name())
-        .version(crate_version!())
+        .version(uucore::crate_version!())
         .about(ABOUT)
         .override_usage(format_usage(USAGE))
         .after_help(format!(
@@ -646,6 +717,37 @@ pub fn uu_app() -> Command {
                 .value_parser(ShortcutValueParser::new(["never", "auto", "always"]))
                 .help("control creation of sparse files. See below"),
         )
+        .arg(
+            Arg::new(options::SELINUX)
+                .short('Z')
+                .help("set SELinux security context of destination file to default type")
+                .action(ArgAction::SetTrue),
+        )
+        .arg(
+            Arg::new(options::CONTEXT)
+                .long(options::CONTEXT)
+                .value_name("CTX")
+                .value_parser(value_parser!(String))
+                .help(
+                    "like -Z, or if CTX is specified then set the SELinux or SMACK security \
+                    context to CTX",
+                )
+                .num_args(0..=1)
+                .require_equals(true)
+                .default_missing_value(""),
+        )
+        .arg(
+            // The 'g' short flag is modeled after advcpmv
+            // See this repo: https://github.com/jarun/advcpmv
+            Arg::new(options::PROGRESS_BAR)
+                .long(options::PROGRESS_BAR)
+                .short('g')
+                .action(ArgAction::SetTrue)
+                .help(
+                    "Display a progress bar. \n\
+                Note: this feature is not supported by GNU coreutils.",
+                ),
+        )
         // TODO: implement the following args
         .arg(
             Arg::new(options::COPY_CONTENTS)
@@ -654,28 +756,7 @@ pub fn uu_app() -> Command {
                 .help("NotImplemented: copy contents of special files when recursive")
                 .action(ArgAction::SetTrue),
         )
-        .arg(
-            Arg::new(options::CONTEXT)
-                .long(options::CONTEXT)
-                .value_name("CTX")
-                .help(
-                    "NotImplemented: set SELinux security context of destination file to \
-                    default type",
-                ),
-        )
         // END TODO
-        .arg(
-            // The 'g' short flag is modeled after advcpmv
-            // See this repo: https://github.com/jarun/advcpmv
-            Arg::new(options::PROGRESS_BAR)
-                .long(options::PROGRESS_BAR)
-                .short('g')
-                .action(clap::ArgAction::SetTrue)
-                .help(
-                    "Display a progress bar. \n\
-                Note: this feature is not supported by GNU coreutils.",
-                ),
-        )
         .arg(
             Arg::new(options::PATHS)
                 .action(ArgAction::Append)
@@ -704,7 +785,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     } else if let Ok(mut matches) = matches {
         let options = Options::from_matches(&matches)?;
 
-        if options.overwrite == OverwriteMode::NoClobber && options.backup != BackupMode::NoBackup {
+        if options.overwrite == OverwriteMode::NoClobber && options.backup != BackupMode::None {
             return Err(UUsageError::new(
                 EXIT_ERR,
                 "options --backup and --no-clobber are mutually exclusive",
@@ -724,7 +805,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
                 // code should still be EXIT_ERR as does GNU cp
                 Error::NotAllFilesCopied => {}
                 // Else we caught a fatal bubbled-up error, log it to stderr
-                _ => show_error!("{}", error),
+                _ => show_error!("{error}"),
             };
             set_exit_code(EXIT_ERR);
         }
@@ -908,7 +989,6 @@ impl Options {
         let not_implemented_opts = vec![
             #[cfg(not(any(windows, unix)))]
             options::ONE_FILE_SYSTEM,
-            options::CONTEXT,
             #[cfg(windows)]
             options::FORCE,
         ];
@@ -930,6 +1010,16 @@ impl Options {
         };
         let update_mode = update_control::determine_update_mode(matches);
 
+        if backup_mode != BackupMode::None
+            && matches
+                .get_one::<String>(update_control::arguments::OPT_UPDATE)
+                .is_some_and(|v| v == "none" || v == "none-fail")
+        {
+            return Err(Error::InvalidArgument(
+                "--backup is mutually exclusive with -n or --update=none-fail".to_string(),
+            ));
+        }
+
         let backup_suffix = backup_control::determine_backup_suffix(matches);
 
         let overwrite = OverwriteMode::from_matches(matches);
@@ -945,7 +1035,6 @@ impl Options {
                 return Err(Error::NotADirectory(dir.clone()));
             }
         };
-
         // cp follows POSIX conventions for overriding options such as "-a",
         // "-d", "--preserve", and "--no-preserve". We can use clap's
         // override-all behavior to achieve this, but there's a challenge: when
@@ -1028,7 +1117,7 @@ impl Options {
             }
         }
 
-        #[cfg(not(feature = "feat_selinux"))]
+        #[cfg(not(feature = "selinux"))]
         if let Preserve::Yes { required } = attributes.context {
             let selinux_disabled_error =
                 Error::Error("SELinux was not enabled during the compile time!".to_string());
@@ -1039,6 +1128,15 @@ impl Options {
             }
         }
 
+        // Extract the SELinux related flags and options
+        let set_selinux_context = matches.get_flag(options::SELINUX);
+
+        let context = if matches.contains_id(options::CONTEXT) {
+            matches.get_one::<String>(options::CONTEXT).cloned()
+        } else {
+            None
+        };
+
         let options = Self {
             attributes_only: matches.get_flag(options::ATTRIBUTES_ONLY),
             copy_contents: matches.get_flag(options::COPY_CONTENTS),
@@ -1072,18 +1170,7 @@ impl Options {
                         }
                     }
                 } else {
-                    #[cfg(any(target_os = "linux", target_os = "android", target_os = "macos"))]
-                    {
-                        ReflinkMode::Auto
-                    }
-                    #[cfg(not(any(
-                        target_os = "linux",
-                        target_os = "android",
-                        target_os = "macos"
-                    )))]
-                    {
-                        ReflinkMode::Never
-                    }
+                    ReflinkMode::default()
                 }
             },
             sparse_mode: {
@@ -1110,6 +1197,8 @@ impl Options {
             recursive,
             target_dir,
             progress_bar: matches.get_flag(options::PROGRESS_BAR),
+            set_selinux_context: set_selinux_context || context.is_some(),
+            context,
         };
 
         Ok(options)
@@ -1170,13 +1259,17 @@ fn parse_path_args(
         return Err("missing file operand".into());
     } else if paths.len() == 1 && options.target_dir.is_none() {
         // Only one file specified
-        return Err(format!("missing destination file operand after {:?}", paths[0]).into());
+        return Err(format!(
+            "missing destination file operand after {}",
+            paths[0].display().to_string().quote()
+        )
+        .into());
     }
 
     // Return an error if the user requested to copy more than one
     // file source to a file target
     if options.no_target_dir && options.target_dir.is_none() && paths.len() > 2 {
-        return Err(format!("extra operand {:?}", paths[2]).into());
+        return Err(format!("extra operand {:}", paths[2].display().to_string().quote()).into());
     }
 
     let target = match options.target_dir {
@@ -1217,7 +1310,7 @@ fn show_error_if_needed(error: &Error) {
             // should return an error from GNU 9.2
         }
         _ => {
-            show_error!("{}", error);
+            show_error!("{error}");
         }
     }
 }
@@ -1265,7 +1358,7 @@ pub fn copy(sources: &[PathBuf], target: &Path, options: &Options) -> CopyResult
 
     for source in sources {
         let normalized_source = normalize_path(source);
-        if options.backup == BackupMode::NoBackup && seen_sources.contains(&normalized_source) {
+        if options.backup == BackupMode::None && seen_sources.contains(&normalized_source) {
             let file_type = if source.symlink_metadata()?.file_type().is_dir() {
                 "directory"
             } else {
@@ -1287,9 +1380,7 @@ pub fn copy(sources: &[PathBuf], target: &Path, options: &Options) -> CopyResult
                 || matches!(options.copy_mode, CopyMode::SymLink)
             {
                 // There is already a file and it isn't a symlink (managed in a different place)
-                if copied_destinations.contains(&dest)
-                    && options.backup != BackupMode::NumberedBackup
-                {
+                if copied_destinations.contains(&dest) && options.backup != BackupMode::Numbered {
                     // If the target file was already created in this cp call, do not overwrite
                     return Err(Error::Error(format!(
                         "will not overwrite just-created '{}' with '{}'",
@@ -1300,7 +1391,7 @@ pub fn copy(sources: &[PathBuf], target: &Path, options: &Options) -> CopyResult
             }
 
             if let Err(error) = copy_source(
-                &progress_bar,
+                progress_bar.as_ref(),
                 source,
                 target,
                 target_type,
@@ -1367,7 +1458,7 @@ fn construct_dest_path(
 }
 #[allow(clippy::too_many_arguments)]
 fn copy_source(
-    progress_bar: &Option<ProgressBar>,
+    progress_bar: Option<&ProgressBar>,
     source: &Path,
     target: &Path,
     target_type: TargetType,
@@ -1432,7 +1523,7 @@ fn file_mode_for_interactive_overwrite(
     {
         #[cfg(unix)]
         {
-            use libc::{mode_t, S_IWUSR};
+            use libc::{S_IWUSR, mode_t};
             use std::os::unix::prelude::MetadataExt;
 
             match path.metadata() {
@@ -1516,6 +1607,42 @@ fn handle_preserve<F: Fn() -> CopyResult<()>>(p: &Preserve, f: F) -> CopyResult<
     Ok(())
 }
 
+/// Copies extended attributes (xattrs) from `source` to `dest`, ensuring that `dest` is temporarily
+/// user-writable if needed and restoring its original permissions afterward. This avoids “Operation
+/// not permitted” errors on read-only files. Returns an error if permission or metadata operations fail,
+/// or if xattr copying fails.
+#[cfg(all(unix, not(target_os = "android")))]
+fn copy_extended_attrs(source: &Path, dest: &Path) -> CopyResult<()> {
+    let metadata = fs::symlink_metadata(dest)?;
+
+    // Check if the destination file is currently read-only for the user.
+    let mut perms = metadata.permissions();
+    let was_readonly = perms.readonly();
+
+    // Temporarily grant user write if it was read-only.
+    if was_readonly {
+        #[allow(clippy::permissions_set_readonly_false)]
+        perms.set_readonly(false);
+        fs::set_permissions(dest, perms)?;
+    }
+
+    // Perform the xattr copy and capture any potential error,
+    // so we can restore permissions before returning.
+    let copy_xattrs_result = copy_xattrs(source, dest);
+
+    // Restore read-only if we changed it.
+    if was_readonly {
+        let mut revert_perms = fs::symlink_metadata(dest)?.permissions();
+        revert_perms.set_readonly(true);
+        fs::set_permissions(dest, revert_perms)?;
+    }
+
+    // If copying xattrs failed, propagate that error now.
+    copy_xattrs_result?;
+
+    Ok(())
+}
+
 /// Copy the specified attributes from one path to another.
 pub(crate) fn copy_attributes(
     source: &Path,
@@ -1529,9 +1656,9 @@ pub(crate) fn copy_attributes(
     #[cfg(unix)]
     handle_preserve(&attributes.ownership, || -> CopyResult<()> {
         use std::os::unix::prelude::MetadataExt;
-        use uucore::perms::wrap_chown;
         use uucore::perms::Verbosity;
         use uucore::perms::VerbosityLevel;
+        use uucore::perms::wrap_chown;
 
         let dest_uid = source_metadata.uid();
         let dest_gid = source_metadata.gid();
@@ -1580,37 +1707,31 @@ pub(crate) fn copy_attributes(
         Ok(())
     })?;
 
-    #[cfg(feature = "feat_selinux")]
+    #[cfg(feature = "selinux")]
     handle_preserve(&attributes.context, || -> CopyResult<()> {
-        let context = selinux::SecurityContext::of_path(source, false, false).map_err(|e| {
-            format!(
-                "failed to get security context of {}: {}",
-                source.display(),
-                e
-            )
-        })?;
-        if let Some(context) = context {
-            context.set_for_path(dest, false, false).map_err(|e| {
-                format!(
-                    "failed to set security context for {}: {}",
-                    dest.display(),
-                    e
-                )
-            })?;
+        // Get the source context and apply it to the destination
+        if let Ok(context) = selinux::SecurityContext::of_path(source, false, false) {
+            if let Some(context) = context {
+                if let Err(e) = context.set_for_path(dest, false, false) {
+                    return Err(Error::Error(format!(
+                        "failed to set the security context of {}: {e}",
+                        dest.display()
+                    )));
+                }
+            }
+        } else {
+            return Err(Error::Error(format!(
+                "failed to get security context of {}",
+                source.display()
+            )));
         }
-
         Ok(())
     })?;
 
     handle_preserve(&attributes.xattr, || -> CopyResult<()> {
         #[cfg(all(unix, not(target_os = "android")))]
         {
-            let xattrs = xattr::list(source)?;
-            for attr in xattrs {
-                if let Some(attr_value) = xattr::get(source, attr.clone())? {
-                    xattr::set(dest, attr, &attr_value[..])?;
-                }
-            }
+            copy_extended_attrs(source, dest)?;
         }
         #[cfg(not(all(unix, not(target_os = "android"))))]
         {
@@ -1696,7 +1817,7 @@ fn is_forbidden_to_copy_to_same_file(
     if !paths_refer_to_same_file(source, dest, dereference_to_compare) {
         return false;
     }
-    if options.backup != BackupMode::NoBackup {
+    if options.backup != BackupMode::None {
         if options.force() && !source_is_symlink {
             return false;
         }
@@ -1716,7 +1837,13 @@ fn is_forbidden_to_copy_to_same_file(
     if options.copy_mode == CopyMode::SymLink && dest_is_symlink {
         return false;
     }
-    if dest_is_symlink && source_is_symlink && !options.dereference {
+    // If source and dest are both the same symlink but with different names, then allow the copy.
+    // This can occur, for example, if source and dest are both hardlinks to the same symlink.
+    if dest_is_symlink
+        && source_is_symlink
+        && source.file_name() != dest.file_name()
+        && !options.dereference
+    {
         return false;
     }
     true
@@ -1736,7 +1863,14 @@ fn handle_existing_dest(
         return Err(format!("{} and {} are the same file", source.quote(), dest.quote()).into());
     }
 
-    if options.update != UpdateMode::ReplaceIfOlder {
+    if options.update == UpdateMode::None {
+        if options.debug {
+            println!("skipped {}", dest.quote());
+        }
+        return Err(Error::Skipped(false));
+    }
+
+    if options.update != UpdateMode::IfOlder {
         options.overwrite.verify(dest, options.debug)?;
     }
 
@@ -1880,7 +2014,7 @@ fn aligned_ancestors<'a>(source: &'a Path, dest: &'a Path) -> Vec<(&'a Path, &'a
 
 fn print_verbose_output(
     parents: bool,
-    progress_bar: &Option<ProgressBar>,
+    progress_bar: Option<&ProgressBar>,
     source: &Path,
     dest: &Path,
 ) {
@@ -1920,6 +2054,7 @@ fn print_paths(parents: bool, source: &Path, dest: &Path) {
 ///
 /// * `Ok(())` - The file was copied successfully.
 /// * `Err(CopyError)` - An error occurred while copying the file.
+#[allow(clippy::too_many_arguments)]
 fn handle_copy_mode(
     source: &Path,
     dest: &Path,
@@ -1928,15 +2063,10 @@ fn handle_copy_mode(
     source_metadata: &Metadata,
     symlinked_files: &mut HashSet<FileInformation>,
     source_in_command_line: bool,
-) -> CopyResult<()> {
-    let source_file_type = source_metadata.file_type();
-
-    let source_is_symlink = source_file_type.is_symlink();
-
-    #[cfg(unix)]
-    let source_is_fifo = source_file_type.is_fifo();
-    #[cfg(not(unix))]
-    let source_is_fifo = false;
+    source_is_fifo: bool,
+    #[cfg(unix)] source_is_stream: bool,
+) -> CopyResult<PerformedAction> {
+    let source_is_symlink = source_metadata.is_symlink();
 
     match options.copy_mode {
         CopyMode::Link => {
@@ -1973,6 +2103,8 @@ fn handle_copy_mode(
                 source_is_symlink,
                 source_is_fifo,
                 symlinked_files,
+                #[cfg(unix)]
+                source_is_stream,
             )?;
         }
         CopyMode::SymLink => {
@@ -1984,7 +2116,7 @@ fn handle_copy_mode(
         CopyMode::Update => {
             if dest.exists() {
                 match options.update {
-                    update_control::UpdateMode::ReplaceAll => {
+                    UpdateMode::All => {
                         copy_helper(
                             source,
                             dest,
@@ -1993,25 +2125,27 @@ fn handle_copy_mode(
                             source_is_symlink,
                             source_is_fifo,
                             symlinked_files,
+                            #[cfg(unix)]
+                            source_is_stream,
                         )?;
                     }
-                    update_control::UpdateMode::ReplaceNone => {
+                    UpdateMode::None => {
                         if options.debug {
                             println!("skipped {}", dest.quote());
                         }
 
-                        return Ok(());
+                        return Ok(PerformedAction::Skipped);
                     }
-                    update_control::UpdateMode::ReplaceNoneFail => {
+                    UpdateMode::NoneFail => {
                         return Err(Error::Error(format!("not replacing '{}'", dest.display())));
                     }
-                    update_control::UpdateMode::ReplaceIfOlder => {
+                    UpdateMode::IfOlder => {
                         let dest_metadata = fs::symlink_metadata(dest)?;
 
                         let src_time = source_metadata.modified()?;
                         let dest_time = dest_metadata.modified()?;
                         if src_time <= dest_time {
-                            return Ok(());
+                            return Ok(PerformedAction::Skipped);
                         } else {
                             options.overwrite.verify(dest, options.debug)?;
 
@@ -2023,6 +2157,8 @@ fn handle_copy_mode(
                                 source_is_symlink,
                                 source_is_fifo,
                                 symlinked_files,
+                                #[cfg(unix)]
+                                source_is_stream,
                             )?;
                         }
                     }
@@ -2036,6 +2172,8 @@ fn handle_copy_mode(
                     source_is_symlink,
                     source_is_fifo,
                     symlinked_files,
+                    #[cfg(unix)]
+                    source_is_stream,
                 )?;
             }
         }
@@ -2049,7 +2187,7 @@ fn handle_copy_mode(
         }
     };
 
-    Ok(())
+    Ok(PerformedAction::Copied)
 }
 
 /// Calculates the permissions for the destination file in a copy operation.
@@ -2104,7 +2242,7 @@ fn calculate_dest_permissions(
 /// after a successful copy.
 #[allow(clippy::cognitive_complexity, clippy::too_many_arguments)]
 fn copy_file(
-    progress_bar: &Option<ProgressBar>,
+    progress_bar: Option<&ProgressBar>,
     source: &Path,
     dest: &Path,
     options: &Options,
@@ -2158,7 +2296,7 @@ fn copy_file(
                 options.overwrite,
                 OverwriteMode::Clobber(ClobberMode::RemoveDestination)
             )
-            && options.backup == BackupMode::NoBackup
+            && options.backup == BackupMode::None
         {
             fs::remove_file(dest)?;
         }
@@ -2189,7 +2327,7 @@ fn copy_file(
                 if !options.dereference {
                     return Ok(());
                 }
-            } else if options.backup != BackupMode::NoBackup && !dest_is_symlink {
+            } else if options.backup != BackupMode::None && !dest_is_symlink {
                 if source == dest {
                     if !options.force() {
                         return Ok(());
@@ -2201,7 +2339,7 @@ fn copy_file(
         }
         handle_existing_dest(source, dest, options, source_in_command_line, copied_files)?;
         if are_hardlinks_to_same_file(source, dest) {
-            if options.copy_mode == CopyMode::Copy && options.backup != BackupMode::NoBackup {
+            if options.copy_mode == CopyMode::Copy {
                 return Ok(());
             }
             if options.copy_mode == CopyMode::Link && (!source_is_symlink || !dest_is_symlink) {
@@ -2224,10 +2362,6 @@ fn copy_file(
         .into());
     }
 
-    if options.verbose {
-        print_verbose_output(options.parents, progress_bar, source, dest);
-    }
-
     if options.preserve_hard_links() {
         // if we encounter a matching device/inode pair in the source tree
         // we can arrange to create a hard link between the corresponding names
@@ -2236,7 +2370,12 @@ fn copy_file(
             &FileInformation::from_path(source, options.dereference(source_in_command_line))
                 .context(format!("cannot stat {}", source.quote()))?,
         ) {
-            std::fs::hard_link(new_source, dest)?;
+            fs::hard_link(new_source, dest)?;
+
+            if options.verbose {
+                print_verbose_output(options.parents, progress_bar, source, dest);
+            }
+
             return Ok(());
         };
     }
@@ -2262,7 +2401,19 @@ fn copy_file(
 
     let dest_permissions = calculate_dest_permissions(dest, &source_metadata, options, context)?;
 
-    handle_copy_mode(
+    #[cfg(unix)]
+    let source_is_fifo = source_metadata.file_type().is_fifo();
+    #[cfg(not(unix))]
+    let source_is_fifo = false;
+
+    #[cfg(unix)]
+    let source_is_stream = source_is_fifo
+        || source_metadata.file_type().is_char_device()
+        || source_metadata.file_type().is_block_device();
+    #[cfg(not(unix))]
+    let source_is_stream = false;
+
+    let performed_action = handle_copy_mode(
         source,
         dest,
         options,
@@ -2270,8 +2421,15 @@ fn copy_file(
         &source_metadata,
         symlinked_files,
         source_in_command_line,
+        source_is_fifo,
+        #[cfg(unix)]
+        source_is_stream,
     )?;
 
+    if options.verbose && performed_action != PerformedAction::Skipped {
+        print_verbose_output(options.parents, progress_bar, source, dest);
+    }
+
     // TODO: implement something similar to gnu's lchown
     if !dest_is_symlink {
         // Here, to match GNU semantics, we quietly ignore an error
@@ -2285,12 +2443,29 @@ fn copy_file(
 
     if options.dereference(source_in_command_line) {
         if let Ok(src) = canonicalize(source, MissingHandling::Normal, ResolveMode::Physical) {
-            copy_attributes(&src, dest, &options.attributes)?;
+            if src.exists() {
+                copy_attributes(&src, dest, &options.attributes)?;
+            }
         }
+    } else if source_is_stream && source.exists() {
+        // Some stream files may not exist after we have copied it,
+        // like anonymous pipes. Thus, we can't really copy its
+        // attributes. However, this is already handled in the stream
+        // copy function (see `copy_stream` under platform/linux.rs).
     } else {
         copy_attributes(source, dest, &options.attributes)?;
     }
 
+    #[cfg(feature = "selinux")]
+    if options.set_selinux_context && uucore::selinux::is_selinux_enabled() {
+        // Set the given selinux permissions on the copied file.
+        if let Err(e) =
+            uucore::selinux::set_selinux_security_context(dest, options.context.as_ref())
+        {
+            return Err(Error::Error(format!("SELinux error: {}", e)));
+        }
+    }
+
     copied_files.insert(
         FileInformation::from_path(source, options.dereference(source_in_command_line))?,
         dest.to_path_buf(),
@@ -2320,10 +2495,10 @@ fn handle_no_preserve_mode(options: &Options, org_mode: u32) -> u32 {
         {
             const MODE_RW_UGO: u32 = S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH;
             const S_IRWXUGO: u32 = S_IRWXU | S_IRWXG | S_IRWXO;
-            if is_explicit_no_preserve_mode {
-                return MODE_RW_UGO;
+            return if is_explicit_no_preserve_mode {
+                MODE_RW_UGO
             } else {
-                return org_mode & S_IRWXUGO;
+                org_mode & S_IRWXUGO
             };
         }
 
@@ -2350,6 +2525,7 @@ fn handle_no_preserve_mode(options: &Options, org_mode: u32) -> u32 {
 
 /// Copy the file from `source` to `dest` either using the normal `fs::copy` or a
 /// copy-on-write scheme if --reflink is specified and the filesystem supports it.
+#[allow(clippy::too_many_arguments)]
 fn copy_helper(
     source: &Path,
     dest: &Path,
@@ -2358,6 +2534,7 @@ fn copy_helper(
     source_is_symlink: bool,
     source_is_fifo: bool,
     symlinked_files: &mut HashSet<FileInformation>,
+    #[cfg(unix)] source_is_stream: bool,
 ) -> CopyResult<()> {
     if options.parents {
         let parent = dest.parent().unwrap_or(dest);
@@ -2368,12 +2545,7 @@ fn copy_helper(
         return Err(Error::NotADirectory(dest.to_path_buf()));
     }
 
-    if source.as_os_str() == "/dev/null" {
-        /* workaround a limitation of fs::copy
-         * https://github.com/rust-lang/rust/issues/79390
-         */
-        File::create(dest).context(dest.display().to_string())?;
-    } else if source_is_fifo && options.recursive && !options.copy_contents {
+    if source_is_fifo && options.recursive && !options.copy_contents {
         #[cfg(unix)]
         copy_fifo(dest, options.overwrite, options.debug)?;
     } else if source_is_symlink {
@@ -2385,8 +2557,10 @@ fn copy_helper(
             options.reflink_mode,
             options.sparse_mode,
             context,
-            #[cfg(any(target_os = "linux", target_os = "android", target_os = "macos"))]
+            #[cfg(unix)]
             source_is_fifo,
+            #[cfg(unix)]
+            source_is_stream,
         )?;
 
         if !options.attributes_only && options.debug {
@@ -2406,12 +2580,7 @@ fn copy_fifo(dest: &Path, overwrite: OverwriteMode, debug: bool) -> CopyResult<(
         fs::remove_file(dest)?;
     }
 
-    let name = CString::new(dest.as_os_str().as_bytes()).unwrap();
-    let err = unsafe { mkfifo(name.as_ptr(), 0o666) };
-    if err == -1 {
-        return Err(format!("cannot create fifo {}: File exists", dest.quote()).into());
-    }
-    Ok(())
+    make_fifo(dest).map_err(|_| format!("cannot create fifo {}: File exists", dest.quote()).into())
 }
 
 fn copy_link(
@@ -2499,7 +2668,7 @@ fn disk_usage_directory(p: &Path) -> io::Result<u64> {
 #[cfg(test)]
 mod tests {
 
-    use crate::{aligned_ancestors, localize_to_target, Attributes, Preserve};
+    use crate::{Attributes, Preserve, aligned_ancestors, localize_to_target};
     use std::path::Path;
 
     #[test]
