@@ -6,27 +6,25 @@
 // spell-checker:ignore (ToDO) chdir execvp progname subcommand subcommands unsets setenv putenv spawnp SIGSEGV SIGBUS sigaction
 
 pub mod native_int_str;
-pub mod parse_error;
 pub mod split_iterator;
 pub mod string_expander;
 pub mod string_parser;
 pub mod variable_parser;
 
 use clap::builder::ValueParser;
-use clap::{crate_name, crate_version, Arg, ArgAction, Command};
+use clap::{Arg, ArgAction, Command, crate_name};
 use ini::Ini;
 use native_int_str::{
-    from_native_int_representation_owned, Convert, NCvt, NativeIntStr, NativeIntString, NativeStr,
+    Convert, NCvt, NativeIntStr, NativeIntString, NativeStr, from_native_int_representation_owned,
 };
 #[cfg(unix)]
 use nix::sys::signal::{
-    raise, sigaction, signal, SaFlags, SigAction, SigHandler, SigHandler::SigIgn, SigSet, Signal,
+    SaFlags, SigAction, SigHandler, SigHandler::SigIgn, SigSet, Signal, raise, sigaction, signal,
 };
 use std::borrow::Cow;
 use std::env;
 use std::ffi::{OsStr, OsString};
 use std::io::{self, Write};
-use std::ops::Deref;
 
 #[cfg(unix)]
 use std::os::unix::ffi::OsStrExt;
@@ -40,10 +38,58 @@ use uucore::line_ending::LineEnding;
 use uucore::signals::signal_by_name_or_value;
 use uucore::{format_usage, help_about, help_section, help_usage, show_warning};
 
+use thiserror::Error;
+
+#[derive(Debug, Error, PartialEq)]
+pub enum EnvError {
+    #[error("no terminating quote in -S string")]
+    EnvMissingClosingQuote(usize, char),
+    #[error("invalid backslash at end of string in -S")]
+    EnvInvalidBackslashAtEndOfStringInMinusS(usize, String),
+    #[error("'\\c' must not appear in double-quoted -S string")]
+    EnvBackslashCNotAllowedInDoubleQuotes(usize),
+    #[error("invalid sequence '\\{}' in -S",.1)]
+    EnvInvalidSequenceBackslashXInMinusS(usize, char),
+    #[error("Missing closing brace")]
+    EnvParsingOfVariableMissingClosingBrace(usize),
+    #[error("Missing variable name")]
+    EnvParsingOfMissingVariable(usize),
+    #[error("Missing closing brace after default value at {}",.0)]
+    EnvParsingOfVariableMissingClosingBraceAfterValue(usize),
+    #[error("Unexpected character: '{}', expected variable name must not start with 0..9",.1)]
+    EnvParsingOfVariableUnexpectedNumber(usize, String),
+    #[error("Unexpected character: '{}', expected a closing brace ('}}') or colon (':')",.1)]
+    EnvParsingOfVariableExceptedBraceOrColon(usize, String),
+    #[error("")]
+    EnvReachedEnd,
+    #[error("")]
+    EnvContinueWithDelimiter,
+    #[error("{}{:?}",.0,.1)]
+    EnvInternalError(usize, string_parser::Error),
+}
+
+impl From<string_parser::Error> for EnvError {
+    fn from(value: string_parser::Error) -> Self {
+        EnvError::EnvInternalError(value.peek_position, value)
+    }
+}
+
 const ABOUT: &str = help_about!("env.md");
 const USAGE: &str = help_usage!("env.md");
 const AFTER_HELP: &str = help_section!("after help", "env.md");
 
+mod options {
+    pub const IGNORE_ENVIRONMENT: &str = "ignore-environment";
+    pub const CHDIR: &str = "chdir";
+    pub const NULL: &str = "null";
+    pub const FILE: &str = "file";
+    pub const UNSET: &str = "unset";
+    pub const DEBUG: &str = "debug";
+    pub const SPLIT_STRING: &str = "split-string";
+    pub const ARGV0: &str = "argv0";
+    pub const IGNORE_SIGNAL: &str = "ignore-signal";
+}
+
 const ERROR_MSG_S_SHEBANG: &str = "use -[v]S to pass options in shebang lines";
 
 struct Options<'a> {
@@ -124,20 +170,17 @@ fn parse_signal_opt<'a>(opts: &mut Options<'a>, opt: &'a OsStr) -> UResult<()> {
         .collect();
 
     let mut sig_vec = Vec::with_capacity(signals.len());
-    signals.into_iter().for_each(|sig| {
-        if !(sig.is_empty()) {
+    for sig in signals {
+        if !sig.is_empty() {
             sig_vec.push(sig);
         }
-    });
+    }
     for sig in sig_vec {
-        let sig_str = match sig.to_str() {
-            Some(s) => s,
-            None => {
-                return Err(USimpleError::new(
-                    1,
-                    format!("{}: invalid signal", sig.quote()),
-                ))
-            }
+        let Some(sig_str) = sig.to_str() else {
+            return Err(USimpleError::new(
+                1,
+                format!("{}: invalid signal", sig.quote()),
+            ));
         };
         let sig_val = parse_signal_value(sig_str)?;
         if !opts.ignore_signal.contains(&sig_val) {
@@ -161,12 +204,14 @@ fn load_config_file(opts: &mut Options) -> UResult<()> {
         };
 
         let conf =
-            conf.map_err(|e| USimpleError::new(1, format!("{}: {}", file.maybe_quote(), e)))?;
+            conf.map_err(|e| USimpleError::new(1, format!("{}: {e}", file.maybe_quote())))?;
 
         for (_, prop) in &conf {
             // ignore all INI section lines (treat them as comments)
             for (key, value) in prop {
-                env::set_var(key, value);
+                unsafe {
+                    env::set_var(key, value);
+                }
             }
         }
     }
@@ -176,23 +221,23 @@ fn load_config_file(opts: &mut Options) -> UResult<()> {
 
 pub fn uu_app() -> Command {
     Command::new(crate_name!())
-        .version(crate_version!())
+        .version(uucore::crate_version!())
         .about(ABOUT)
         .override_usage(format_usage(USAGE))
         .after_help(AFTER_HELP)
         .infer_long_args(true)
         .trailing_var_arg(true)
         .arg(
-            Arg::new("ignore-environment")
+            Arg::new(options::IGNORE_ENVIRONMENT)
                 .short('i')
-                .long("ignore-environment")
+                .long(options::IGNORE_ENVIRONMENT)
                 .help("start with an empty environment")
                 .action(ArgAction::SetTrue),
         )
         .arg(
-            Arg::new("chdir")
+            Arg::new(options::CHDIR)
                 .short('C') // GNU env compatibility
-                .long("chdir")
+                .long(options::CHDIR)
                 .number_of_values(1)
                 .value_name("DIR")
                 .value_parser(ValueParser::os_string())
@@ -200,9 +245,9 @@ pub fn uu_app() -> Command {
                 .help("change working directory to DIR"),
         )
         .arg(
-            Arg::new("null")
+            Arg::new(options::NULL)
                 .short('0')
-                .long("null")
+                .long(options::NULL)
                 .help(
                     "end each output line with a 0 byte rather than a newline (only \
                 valid when printing the environment)",
@@ -210,9 +255,9 @@ pub fn uu_app() -> Command {
                 .action(ArgAction::SetTrue),
         )
         .arg(
-            Arg::new("file")
+            Arg::new(options::FILE)
                 .short('f')
-                .long("file")
+                .long(options::FILE)
                 .value_name("PATH")
                 .value_hint(clap::ValueHint::FilePath)
                 .value_parser(ValueParser::os_string())
@@ -223,34 +268,34 @@ pub fn uu_app() -> Command {
                 ),
         )
         .arg(
-            Arg::new("unset")
+            Arg::new(options::UNSET)
                 .short('u')
-                .long("unset")
+                .long(options::UNSET)
                 .value_name("NAME")
                 .action(ArgAction::Append)
                 .value_parser(ValueParser::os_string())
                 .help("remove variable from the environment"),
         )
         .arg(
-            Arg::new("debug")
+            Arg::new(options::DEBUG)
                 .short('v')
-                .long("debug")
+                .long(options::DEBUG)
                 .action(ArgAction::Count)
                 .help("print verbose information for each processing step"),
         )
         .arg(
-            Arg::new("split-string") // split string handling is implemented directly, not using CLAP. But this entry here is needed for the help information output.
+            Arg::new(options::SPLIT_STRING) // split string handling is implemented directly, not using CLAP. But this entry here is needed for the help information output.
                 .short('S')
-                .long("split-string")
+                .long(options::SPLIT_STRING)
                 .value_name("S")
                 .action(ArgAction::Set)
                 .value_parser(ValueParser::os_string())
                 .help("process and split S into separate arguments; used to pass multiple arguments on shebang lines")
         ).arg(
-            Arg::new("argv0")
-                .overrides_with("argv0")
+            Arg::new(options::ARGV0)
+                .overrides_with(options::ARGV0)
                 .short('a')
-                .long("argv0")
+                .long(options::ARGV0)
                 .value_name("a")
                 .action(ArgAction::Set)
                 .value_parser(ValueParser::os_string())
@@ -263,8 +308,8 @@ pub fn uu_app() -> Command {
                 .value_parser(ValueParser::os_string())
         )
         .arg(
-            Arg::new("ignore-signal")
-                .long("ignore-signal")
+            Arg::new(options::IGNORE_SIGNAL)
+                .long(options::IGNORE_SIGNAL)
                 .value_name("SIG")
                 .action(ArgAction::Append)
                 .value_parser(ValueParser::os_string())
@@ -274,20 +319,28 @@ pub fn uu_app() -> Command {
 
 pub fn parse_args_from_str(text: &NativeIntStr) -> UResult<Vec<NativeIntString>> {
     split_iterator::split(text).map_err(|e| match e {
-        parse_error::ParseError::BackslashCNotAllowedInDoubleQuotes { pos: _ } => {
-            USimpleError::new(125, "'\\c' must not appear in double-quoted -S string")
+        EnvError::EnvBackslashCNotAllowedInDoubleQuotes(_) => USimpleError::new(125, e.to_string()),
+        EnvError::EnvInvalidBackslashAtEndOfStringInMinusS(_, _) => {
+            USimpleError::new(125, e.to_string())
         }
-        parse_error::ParseError::InvalidBackslashAtEndOfStringInMinusS { pos: _, quoting: _ } => {
-            USimpleError::new(125, "invalid backslash at end of string in -S")
+        EnvError::EnvInvalidSequenceBackslashXInMinusS(_, _) => {
+            USimpleError::new(125, e.to_string())
         }
-        parse_error::ParseError::InvalidSequenceBackslashXInMinusS { pos: _, c } => {
-            USimpleError::new(125, format!("invalid sequence '\\{c}' in -S"))
+        EnvError::EnvMissingClosingQuote(_, _) => USimpleError::new(125, e.to_string()),
+        EnvError::EnvParsingOfVariableMissingClosingBrace(pos) => {
+            USimpleError::new(125, format!("variable name issue (at {pos}): {e}"))
         }
-        parse_error::ParseError::MissingClosingQuote { pos: _, c: _ } => {
-            USimpleError::new(125, "no terminating quote in -S string")
+        EnvError::EnvParsingOfMissingVariable(pos) => {
+            USimpleError::new(125, format!("variable name issue (at {pos}): {e}"))
         }
-        parse_error::ParseError::ParsingOfVariableNameFailed { pos, msg } => {
-            USimpleError::new(125, format!("variable name issue (at {pos}): {msg}",))
+        EnvError::EnvParsingOfVariableMissingClosingBraceAfterValue(pos) => {
+            USimpleError::new(125, format!("variable name issue (at {pos}): {e}"))
+        }
+        EnvError::EnvParsingOfVariableUnexpectedNumber(pos, _) => {
+            USimpleError::new(125, format!("variable name issue (at {pos}): {e}"))
+        }
+        EnvError::EnvParsingOfVariableExceptedBraceOrColon(pos, _) => {
+            USimpleError::new(125, format!("variable name issue (at {pos}): {e}"))
         }
         _ => USimpleError::new(125, format!("Error: {e:?}")),
     })
@@ -296,14 +349,14 @@ pub fn parse_args_from_str(text: &NativeIntStr) -> UResult<Vec<NativeIntString>>
 fn debug_print_args(args: &[OsString]) {
     eprintln!("input args:");
     for (i, arg) in args.iter().enumerate() {
-        eprintln!("arg[{}]: {}", i, arg.quote());
+        eprintln!("arg[{i}]: {}", arg.quote());
     }
 }
 
 fn check_and_handle_string_args(
     arg: &OsString,
     prefix_to_test: &str,
-    all_args: &mut Vec<std::ffi::OsString>,
+    all_args: &mut Vec<OsString>,
     do_debug_print_args: Option<&Vec<OsString>>,
 ) -> UResult<bool> {
     let native_arg = NCvt::convert(arg);
@@ -336,7 +389,7 @@ impl EnvAppData {
     fn make_error_no_such_file_or_dir(&self, prog: &OsStr) -> Box<dyn UError> {
         uucore::show_error!("{}: No such file or directory", prog.quote());
         if !self.had_string_argument {
-            uucore::show_error!("{}", ERROR_MSG_S_SHEBANG);
+            uucore::show_error!("{ERROR_MSG_S_SHEBANG}");
         }
         ExitCode::new(127)
     }
@@ -344,9 +397,33 @@ impl EnvAppData {
     fn process_all_string_arguments(
         &mut self,
         original_args: &Vec<OsString>,
-    ) -> UResult<Vec<std::ffi::OsString>> {
-        let mut all_args: Vec<std::ffi::OsString> = Vec::new();
-        for arg in original_args {
+    ) -> UResult<Vec<OsString>> {
+        let mut all_args: Vec<OsString> = Vec::new();
+        let mut process_flags = true;
+        let mut expecting_arg = false;
+        // Leave out split-string since it's a special case below
+        let flags_with_args = [
+            options::ARGV0,
+            options::CHDIR,
+            options::FILE,
+            options::IGNORE_SIGNAL,
+            options::UNSET,
+        ];
+        let short_flags_with_args = ['a', 'C', 'f', 'u'];
+        for (n, arg) in original_args.iter().enumerate() {
+            let arg_str = arg.to_string_lossy();
+            // Stop processing env flags once we reach the command or -- argument
+            if 0 < n
+                && !expecting_arg
+                && (arg == "--" || !(arg_str.starts_with('-') || arg_str.contains('=')))
+            {
+                process_flags = false;
+            }
+            if !process_flags {
+                all_args.push(arg.clone());
+                continue;
+            }
+            expecting_arg = false;
             match arg {
                 b if check_and_handle_string_args(b, "--split-string", &mut all_args, None)? => {
                     self.had_string_argument = true;
@@ -370,8 +447,15 @@ impl EnvAppData {
                     self.had_string_argument = true;
                 }
                 _ => {
-                    let arg_str = arg.to_string_lossy();
-
+                    if let Some(flag) = arg_str.strip_prefix("--") {
+                        if flags_with_args.contains(&flag) {
+                            expecting_arg = true;
+                        }
+                    } else if let Some(flag) = arg_str.strip_prefix("-") {
+                        for c in flag.chars() {
+                            expecting_arg = short_flags_with_args.contains(&c);
+                        }
+                    }
                     // Short unset option (-u) is not allowed to contain '='
                     if arg_str.contains('=')
                         && arg_str.starts_with("-u")
@@ -409,10 +493,10 @@ impl EnvAppData {
                         let s = format!("{e}");
                         if !s.is_empty() {
                             let s = s.trim_end();
-                            uucore::show_error!("{}", s);
+                            uucore::show_error!("{s}");
                         }
-                        uucore::show_error!("{}", ERROR_MSG_S_SHEBANG);
-                        uucore::error::ExitCode::new(125)
+                        uucore::show_error!("{ERROR_MSG_S_SHEBANG}");
+                        ExitCode::new(125)
                     }
                 }
             })?;
@@ -503,9 +587,9 @@ impl EnvAppData {
         if do_debug_printing {
             eprintln!("executing: {}", prog.maybe_quote());
             let arg_prefix = "   arg";
-            eprintln!("{}[{}]= {}", arg_prefix, 0, arg0.quote());
+            eprintln!("{arg_prefix}[{}]= {}", 0, arg0.quote());
             for (i, arg) in args.iter().enumerate() {
-                eprintln!("{}[{}]= {}", arg_prefix, i + 1, arg.quote());
+                eprintln!("{arg_prefix}[{}]= {}", i + 1, arg.quote());
             }
         }
 
@@ -539,19 +623,21 @@ impl EnvAppData {
                 }
                 return Err(exit.code().unwrap().into());
             }
-            Err(ref err) => match err.kind() {
-                io::ErrorKind::NotFound | io::ErrorKind::InvalidInput => {
-                    return Err(self.make_error_no_such_file_or_dir(prog.deref()));
-                }
-                io::ErrorKind::PermissionDenied => {
-                    uucore::show_error!("{}: Permission denied", prog.quote());
-                    return Err(126.into());
-                }
-                _ => {
-                    uucore::show_error!("unknown error: {:?}", err);
-                    return Err(126.into());
-                }
-            },
+            Err(ref err) => {
+                return match err.kind() {
+                    io::ErrorKind::NotFound | io::ErrorKind::InvalidInput => {
+                        Err(self.make_error_no_such_file_or_dir(&prog))
+                    }
+                    io::ErrorKind::PermissionDenied => {
+                        uucore::show_error!("{}: Permission denied", prog.quote());
+                        Err(126.into())
+                    }
+                    _ => {
+                        uucore::show_error!("unknown error: {err:?}");
+                        Err(126.into())
+                    }
+                };
+            }
             Ok(_) => (),
         }
         Ok(())
@@ -562,7 +648,9 @@ fn apply_removal_of_all_env_vars(opts: &Options<'_>) {
     // remove all env vars if told to ignore presets
     if opts.ignore_env {
         for (ref name, _) in env::vars_os() {
-            env::remove_var(name);
+            unsafe {
+                env::remove_var(name);
+            }
         }
     }
 }
@@ -637,8 +725,9 @@ fn apply_unset_env_vars(opts: &Options<'_>) -> Result<(), Box<dyn UError>> {
                 format!("cannot unset {}: Invalid argument", name.quote()),
             ));
         }
-
-        env::remove_var(name);
+        unsafe {
+            env::remove_var(name);
+        }
     }
     Ok(())
 }
@@ -695,7 +784,9 @@ fn apply_specified_env_vars(opts: &Options<'_>) {
             show_warning!("no name specified for value {}", val.quote());
             continue;
         }
-        env::set_var(name, val);
+        unsafe {
+            env::set_var(name, val);
+        }
     }
 }
 
@@ -704,7 +795,7 @@ fn apply_ignore_signal(opts: &Options<'_>) -> UResult<()> {
     for &sig_value in &opts.ignore_signal {
         let sig: Signal = (sig_value as i32)
             .try_into()
-            .map_err(|e| std::io::Error::from_raw_os_error(e as i32))?;
+            .map_err(|e| io::Error::from_raw_os_error(e as i32))?;
 
         ignore_signal(sig)?;
     }
@@ -739,7 +830,7 @@ mod tests {
 
     #[test]
     fn test_split_string_environment_vars_test() {
-        std::env::set_var("FOO", "BAR");
+        unsafe { env::set_var("FOO", "BAR") };
         assert_eq!(
             NCvt::convert(vec!["FOO=bar", "sh", "-c", "echo xBARx =$FOO="]),
             parse_args_from_str(&NCvt::convert(r#"FOO=bar sh -c "echo x${FOO}x =\$FOO=""#))
@@ -755,16 +846,80 @@ mod tests {
         );
         assert_eq!(
             NCvt::convert(vec!["A=B", "FOO=AR", "sh", "-c", "echo $A$FOO"]),
-            parse_args_from_str(&NCvt::convert(r#"A=B FOO=AR  sh -c 'echo $A$FOO'"#)).unwrap()
+            parse_args_from_str(&NCvt::convert(r"A=B FOO=AR  sh -c 'echo $A$FOO'")).unwrap()
         );
         assert_eq!(
             NCvt::convert(vec!["A=B", "FOO=AR", "sh", "-c", "echo $A$FOO"]),
-            parse_args_from_str(&NCvt::convert(r#"A=B FOO=AR  sh -c 'echo $A$FOO'"#)).unwrap()
+            parse_args_from_str(&NCvt::convert(r"A=B FOO=AR  sh -c 'echo $A$FOO'")).unwrap()
         );
 
         assert_eq!(
             NCvt::convert(vec!["-i", "A=B ' C"]),
-            parse_args_from_str(&NCvt::convert(r#"-i A='B \' C'"#)).unwrap()
+            parse_args_from_str(&NCvt::convert(r"-i A='B \' C'")).unwrap()
         );
     }
+
+    #[test]
+    fn test_error_cases() {
+        // Test EnvBackslashCNotAllowedInDoubleQuotes
+        let result = parse_args_from_str(&NCvt::convert(r#"sh -c "echo \c""#));
+        assert!(result.is_err());
+        assert_eq!(
+            result.unwrap_err().to_string(),
+            "'\\c' must not appear in double-quoted -S string"
+        );
+
+        // Test EnvInvalidBackslashAtEndOfStringInMinusS
+        let result = parse_args_from_str(&NCvt::convert(r#"sh -c "echo \"#));
+        assert!(result.is_err());
+        assert_eq!(
+            result.unwrap_err().to_string(),
+            "no terminating quote in -S string"
+        );
+
+        // Test EnvInvalidSequenceBackslashXInMinusS
+        let result = parse_args_from_str(&NCvt::convert(r#"sh -c "echo \x""#));
+        assert!(result.is_err());
+        assert!(
+            result
+                .unwrap_err()
+                .to_string()
+                .contains("invalid sequence '\\x' in -S")
+        );
+
+        // Test EnvMissingClosingQuote
+        let result = parse_args_from_str(&NCvt::convert(r#"sh -c "echo "#));
+        assert!(result.is_err());
+        assert_eq!(
+            result.unwrap_err().to_string(),
+            "no terminating quote in -S string"
+        );
+
+        // Test variable-related errors
+        let result = parse_args_from_str(&NCvt::convert(r"echo ${FOO"));
+        assert!(result.is_err());
+        assert!(
+            result
+                .unwrap_err()
+                .to_string()
+                .contains("variable name issue (at 10): Missing closing brace")
+        );
+
+        let result = parse_args_from_str(&NCvt::convert(r"echo ${FOO:-value"));
+        assert!(result.is_err());
+        assert!(
+            result
+                .unwrap_err()
+                .to_string()
+                .contains("variable name issue (at 17): Missing closing brace after default value")
+        );
+
+        let result = parse_args_from_str(&NCvt::convert(r"echo ${1FOO}"));
+        assert!(result.is_err());
+        assert!(result.unwrap_err().to_string().contains("variable name issue (at 7): Unexpected character: '1', expected variable name must not start with 0..9"));
+
+        let result = parse_args_from_str(&NCvt::convert(r"echo ${FOO?}"));
+        assert!(result.is_err());
+        assert!(result.unwrap_err().to_string().contains("variable name issue (at 10): Unexpected character: '?', expected a closing brace ('}') or colon (':')"));
+    }
 }
