@@ -5,23 +5,21 @@
 
 // spell-checker:ignore (ToDO) retcode
 
-use clap::{
-    builder::NonEmptyStringValueParser, crate_version, Arg, ArgAction, ArgMatches, Command,
-};
+use clap::{Arg, ArgAction, ArgMatches, Command, builder::NonEmptyStringValueParser};
 use std::{
-    io::{stdout, Write},
+    io::{Write, stdout},
     path::{Path, PathBuf},
 };
 use uucore::fs::make_path_relative_to;
 use uucore::{
-    display::{print_verbatim, Quotable},
-    error::{FromIo, UResult},
+    display::{Quotable, print_verbatim},
+    error::{FromIo, UClapError, UResult},
     format_usage,
-    fs::{canonicalize, MissingHandling, ResolveMode},
+    fs::{MissingHandling, ResolveMode, canonicalize},
     help_about, help_usage,
     line_ending::LineEnding,
+    show_if_err,
 };
-use uucore::{error::UClapError, show, show_if_err};
 
 static ABOUT: &str = help_about!("realpath.md");
 const USAGE: &str = help_usage!("realpath.md");
@@ -90,7 +88,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
 pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .version(crate_version!())
+        .version(uucore::crate_version!())
         .about(ABOUT)
         .override_usage(format_usage(USAGE))
         .infer_long_args(true)
@@ -266,11 +264,11 @@ fn resolve_path(
 /// Conditionally converts an absolute path to a relative form,
 /// according to the rules:
 /// 1. if only `relative_to` is given, the result is relative to `relative_to`
-/// 1. if only `relative_base` is given, it checks whether given `path` is a descendant
-/// of `relative_base`, on success the result is relative to `relative_base`, otherwise
-/// the result is the given `path`
-/// 1. if both `relative_to` and `relative_base` are given, the result is relative to `relative_to`
-/// if `path` is a descendant of `relative_base`, otherwise the result is `path`
+/// 2. if only `relative_base` is given, it checks whether given `path` is a descendant
+///    of `relative_base`, on success the result is relative to `relative_base`, otherwise
+///    the result is the given `path`
+/// 3. if both `relative_to` and `relative_base` are given, the result is relative to `relative_to`
+///    if `path` is a descendant of `relative_base`, otherwise the result is `path`
 ///
 /// For more information see
 /// <https://www.gnu.org/software/coreutils/manual/html_node/Realpath-usage-examples.html>
