@@ -5,9 +5,9 @@
 
 // spell-checker:ignore (ToDO) sysv
 
-use clap::{crate_version, Arg, ArgAction, Command};
+use clap::{Arg, ArgAction, Command};
 use std::fs::File;
-use std::io::{stdin, Read};
+use std::io::{ErrorKind, Read, Write, stdin, stdout};
 use std::path::Path;
 use uucore::display::Quotable;
 use uucore::error::{FromIo, UResult, USimpleError};
@@ -16,49 +16,46 @@ use uucore::{format_usage, help_about, help_usage, show};
 const USAGE: &str = help_usage!("sum.md");
 const ABOUT: &str = help_about!("sum.md");
 
-// This can be replaced with usize::div_ceil once it is stabilized.
-// This implementation approach is optimized for when `b` is a constant,
-// particularly a power of two.
-const fn div_ceil(a: usize, b: usize) -> usize {
-    (a + b - 1) / b
-}
-
-fn bsd_sum(mut reader: Box<dyn Read>) -> (usize, u16) {
+fn bsd_sum(mut reader: impl Read) -> std::io::Result<(usize, u16)> {
     let mut buf = [0; 4096];
     let mut bytes_read = 0;
     let mut checksum: u16 = 0;
     loop {
         match reader.read(&mut buf) {
-            Ok(n) if n != 0 => {
+            Ok(0) => break,
+            Ok(n) => {
                 bytes_read += n;
-                for &byte in &buf[..n] {
-                    checksum = checksum.rotate_right(1);
-                    checksum = checksum.wrapping_add(u16::from(byte));
-                }
+                checksum = buf[..n].iter().fold(checksum, |acc, &byte| {
+                    let rotated = acc.rotate_right(1);
+                    rotated.wrapping_add(u16::from(byte))
+                });
             }
-            _ => break,
+            Err(e) if e.kind() == ErrorKind::Interrupted => continue,
+            Err(e) => return Err(e),
         }
     }
 
     // Report blocks read in terms of 1024-byte blocks.
-    let blocks_read = div_ceil(bytes_read, 1024);
-    (blocks_read, checksum)
+    let blocks_read = bytes_read.div_ceil(1024);
+    Ok((blocks_read, checksum))
 }
 
-fn sysv_sum(mut reader: Box<dyn Read>) -> (usize, u16) {
+fn sysv_sum(mut reader: impl Read) -> std::io::Result<(usize, u16)> {
     let mut buf = [0; 4096];
     let mut bytes_read = 0;
     let mut ret = 0u32;
 
     loop {
         match reader.read(&mut buf) {
-            Ok(n) if n != 0 => {
+            Ok(0) => break,
+            Ok(n) => {
                 bytes_read += n;
-                for &byte in &buf[..n] {
-                    ret = ret.wrapping_add(u32::from(byte));
-                }
+                ret = buf[..n]
+                    .iter()
+                    .fold(ret, |acc, &byte| acc.wrapping_add(u32::from(byte)));
             }
-            _ => break,
+            Err(e) if e.kind() == ErrorKind::Interrupted => continue,
+            Err(e) => return Err(e),
         }
     }
 
@@ -66,8 +63,8 @@ fn sysv_sum(mut reader: Box<dyn Read>) -> (usize, u16) {
     ret = (ret & 0xffff) + (ret >> 16);
 
     // Report blocks read in terms of 512-byte blocks.
-    let blocks_read = div_ceil(bytes_read, 512);
-    (blocks_read, ret as u16)
+    let blocks_read = bytes_read.div_ceil(512);
+    Ok((blocks_read, ret as u16))
 }
 
 fn open(name: &str) -> UResult<Box<dyn Read>> {
@@ -102,8 +99,6 @@ mod options {
 
 #[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let args = args.collect_lossy();
-
     let matches = uu_app().try_get_matches_from(args)?;
 
     let files: Vec<String> = match matches.get_many::<String>(options::FILE) {
@@ -128,12 +123,13 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
             sysv_sum(reader)
         } else {
             bsd_sum(reader)
-        };
+        }?;
 
+        let mut stdout = stdout().lock();
         if print_names {
-            println!("{sum:0width$} {blocks:width$} {file}");
+            writeln!(stdout, "{sum:0width$} {blocks:width$} {file}")?;
         } else {
-            println!("{sum:0width$} {blocks:width$}");
+            writeln!(stdout, "{sum:0width$} {blocks:width$}")?;
         }
     }
     Ok(())
@@ -141,7 +137,7 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
 pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
-        .version(crate_version!())
+        .version(uucore::crate_version!())
         .override_usage(format_usage(USAGE))
         .about(ABOUT)
         .infer_long_args(true)
