@@ -5,17 +5,15 @@
 
 //! Common functions to manage permissions
 
+// spell-checker:ignore (jargon) TOCTOU
+
 use crate::display::Quotable;
-use crate::error::strip_errno;
-use crate::error::UResult;
-use crate::error::USimpleError;
+use crate::error::{UResult, USimpleError, strip_errno};
 pub use crate::features::entries;
-use crate::fs::resolve_relative_path;
 use crate::show_error;
-use clap::Arg;
-use clap::ArgMatches;
-use clap::Command;
-use libc::{self, gid_t, uid_t};
+use clap::{Arg, ArgMatches, Command};
+use libc::{gid_t, uid_t};
+use options::traverse;
 use walkdir::WalkDir;
 
 use std::io::Error as IOError;
@@ -26,7 +24,7 @@ use std::fs::Metadata;
 use std::os::unix::fs::MetadataExt;
 
 use std::os::unix::ffi::OsStrExt;
-use std::path::Path;
+use std::path::{MAIN_SEPARATOR, Path};
 
 /// The various level of verbosity
 #[derive(PartialEq, Eq, Clone, Debug)]
@@ -36,6 +34,7 @@ pub enum VerbosityLevel {
     Verbose,
     Normal,
 }
+
 #[derive(PartialEq, Eq, Clone, Debug)]
 pub struct Verbosity {
     pub groups_only: bool,
@@ -81,21 +80,19 @@ pub fn wrap_chown<P: AsRef<Path>>(
             VerbosityLevel::Silent => (),
             level => {
                 out = format!(
-                    "changing {} of {}: {}",
+                    "changing {} of {}: {e}",
                     if verbosity.groups_only {
                         "group"
                     } else {
                         "ownership"
                     },
                     path.quote(),
-                    e
                 );
                 if level == VerbosityLevel::Verbose {
                     out = if verbosity.groups_only {
                         let gid = meta.gid();
                         format!(
-                            "{}\nfailed to change group of {} from {} to {}",
-                            out,
+                            "{out}\nfailed to change group of {} from {} to {}",
                             path.quote(),
                             entries::gid2grp(gid).unwrap_or_else(|_| gid.to_string()),
                             entries::gid2grp(dest_gid).unwrap_or_else(|_| dest_gid.to_string())
@@ -104,8 +101,7 @@ pub fn wrap_chown<P: AsRef<Path>>(
                         let uid = meta.uid();
                         let gid = meta.gid();
                         format!(
-                            "{}\nfailed to change ownership of {} from {}:{} to {}:{}",
-                            out,
+                            "{out}\nfailed to change ownership of {} from {}:{} to {}:{}",
                             path.quote(),
                             entries::uid2usr(uid).unwrap_or_else(|_| uid.to_string()),
                             entries::gid2grp(gid).unwrap_or_else(|_| gid.to_string()),
@@ -182,6 +178,7 @@ pub enum TraverseSymlinks {
 pub struct ChownExecutor {
     pub dest_uid: Option<u32>,
     pub dest_gid: Option<u32>,
+    pub raw_owner: String, // The owner of the file as input by the user in the command line.
     pub traverse_symlinks: TraverseSymlinks,
     pub verbosity: Verbosity,
     pub filter: IfFrom,
@@ -191,6 +188,73 @@ pub struct ChownExecutor {
     pub dereference: bool,
 }
 
+#[cfg(test)]
+pub fn check_root(path: &Path, would_recurse_symlink: bool) -> bool {
+    is_root(path, would_recurse_symlink)
+}
+
+/// In the context of chown and chgrp, check whether we are in a "preserve-root" scenario.
+///
+/// In particular, we want to prohibit further traversal only if:
+///     (--preserve-root and -R present) &&
+///     (path canonicalizes to "/") &&
+///     (
+///         (path is a symlink && would traverse/recurse this symlink) ||
+///         (path is not a symlink)
+///     )
+/// The first clause is checked by the caller, the second and third clause is checked here.
+/// The caller has to evaluate -P/-H/-L into 'would_recurse_symlink'.
+/// Recall that canonicalization resolves both relative paths (e.g. "..") and symlinks.
+fn is_root(path: &Path, would_traverse_symlink: bool) -> bool {
+    // The third clause can be evaluated without any syscalls, so we do that first.
+    // If we would_recurse_symlink, then the clause is true no matter whether the path is a symlink
+    // or not. Otherwise, we only need to check here if the path can syntactically be a symlink:
+    if !would_traverse_symlink {
+        // We cannot check path.is_dir() here, as this would resolve symlinks,
+        // which we need to avoid here.
+        // All directory-ish paths match "*/", except ".", "..", "*/.", and "*/..".
+        let path_bytes = path.as_os_str().as_encoded_bytes();
+        let looks_like_dir = path_bytes == [b'.']
+            || path_bytes == [b'.', b'.']
+            || path_bytes.ends_with(&[MAIN_SEPARATOR as u8])
+            || path_bytes.ends_with(&[MAIN_SEPARATOR as u8, b'.'])
+            || path_bytes.ends_with(&[MAIN_SEPARATOR as u8, b'.', b'.']);
+
+        if !looks_like_dir {
+            return false;
+        }
+    }
+
+    // FIXME: TOCTOU bug! canonicalize() runs at a different time than WalkDir's recursion decision.
+    // However, we're forced to make the decision whether to warn about --preserve-root
+    // *before* even attempting to chown the path, let alone doing the stat inside WalkDir.
+    if let Ok(p) = path.canonicalize() {
+        let path_buf = path.to_path_buf();
+        if p.parent().is_none() {
+            if path_buf.as_os_str() == "/" {
+                show_error!("it is dangerous to operate recursively on '/'");
+            } else {
+                show_error!(
+                    "it is dangerous to operate recursively on {} (same as '/')",
+                    path_buf.quote()
+                );
+            }
+            show_error!("use --no-preserve-root to override this failsafe");
+            return true;
+        }
+    }
+
+    false
+}
+
+pub fn get_metadata(file: &Path, follow: bool) -> Result<Metadata, std::io::Error> {
+    if follow {
+        file.metadata()
+    } else {
+        file.symlink_metadata()
+    }
+}
+
 impl ChownExecutor {
     pub fn exec(&self) -> UResult<()> {
         let mut ret = 0;
@@ -203,38 +267,26 @@ impl ChownExecutor {
         Ok(())
     }
 
+    #[allow(clippy::cognitive_complexity)]
     fn traverse<P: AsRef<Path>>(&self, root: P) -> i32 {
         let path = root.as_ref();
-        let meta = match self.obtain_meta(path, self.dereference) {
-            Some(m) => m,
-            _ => return 1,
+        let Some(meta) = self.obtain_meta(path, self.dereference) else {
+            if self.verbosity.level == VerbosityLevel::Verbose {
+                println!(
+                    "failed to change ownership of {} to {}",
+                    path.quote(),
+                    self.raw_owner
+                );
+            }
+            return 1;
         };
 
-        // Prohibit only if:
-        // (--preserve-root and -R present) &&
-        // (
-        //     (argument is not symlink && resolved to be '/') ||
-        //     (argument is symlink && should follow argument && resolved to be '/')
-        // )
-        if self.recursive && self.preserve_root {
-            let may_exist = if self.dereference {
-                path.canonicalize().ok()
-            } else {
-                let real = resolve_relative_path(path);
-                if real.is_dir() {
-                    Some(real.canonicalize().expect("failed to get real path"))
-                } else {
-                    Some(real.into_owned())
-                }
-            };
-
-            if let Some(p) = may_exist {
-                if p.parent().is_none() {
-                    show_error!("it is dangerous to operate recursively on '/'");
-                    show_error!("use --no-preserve-root to override this failsafe");
-                    return 1;
-                }
-            }
+        if self.recursive
+            && self.preserve_root
+            && is_root(path, self.traverse_symlinks != TraverseSymlinks::None)
+        {
+            // Fail-fast, do not attempt to recurse.
+            return 1;
         }
 
         let ret = if self.matched(meta.uid(), meta.gid()) {
@@ -248,18 +300,23 @@ impl ChownExecutor {
             ) {
                 Ok(n) => {
                     if !n.is_empty() {
-                        show_error!("{}", n);
+                        show_error!("{n}");
                     }
                     0
                 }
                 Err(e) => {
                     if self.verbosity.level != VerbosityLevel::Silent {
-                        show_error!("{}", e);
+                        show_error!("{e}");
                     }
                     1
                 }
             }
         } else {
+            self.print_verbose_ownership_retained_as(
+                path,
+                meta.uid(),
+                self.dest_gid.map(|_| meta.gid()),
+            );
             0
         };
 
@@ -270,6 +327,7 @@ impl ChownExecutor {
         }
     }
 
+    #[allow(clippy::cognitive_complexity)]
     fn dive_into<P: AsRef<Path>>(&self, root: P) -> i32 {
         let root = root.as_ref();
 
@@ -299,27 +357,36 @@ impl ChownExecutor {
                             }
                         );
                     } else {
-                        show_error!("{}", e);
+                        show_error!("{e}");
                     }
                     continue;
                 }
                 Ok(entry) => entry,
             };
             let path = entry.path();
-            let meta = match self.obtain_meta(path, self.dereference) {
-                Some(m) => m,
-                _ => {
-                    ret = 1;
-                    if entry.file_type().is_dir() {
-                        // Instruct walkdir to skip this directory to avoid getting another error
-                        // when walkdir tries to query the children of this directory.
-                        iterator.skip_current_dir();
-                    }
-                    continue;
+
+            let Some(meta) = self.obtain_meta(path, self.dereference) else {
+                ret = 1;
+                if entry.file_type().is_dir() {
+                    // Instruct walkdir to skip this directory to avoid getting another error
+                    // when walkdir tries to query the children of this directory.
+                    iterator.skip_current_dir();
                 }
+                continue;
             };
 
+            if self.preserve_root && is_root(path, self.traverse_symlinks == TraverseSymlinks::All)
+            {
+                // Fail-fast, do not recurse further.
+                return 1;
+            }
+
             if !self.matched(meta.uid(), meta.gid()) {
+                self.print_verbose_ownership_retained_as(
+                    path,
+                    meta.uid(),
+                    self.dest_gid.map(|_| meta.gid()),
+                );
                 continue;
             }
 
@@ -333,13 +400,13 @@ impl ChownExecutor {
             ) {
                 Ok(n) => {
                     if !n.is_empty() {
-                        show_error!("{}", n);
+                        show_error!("{n}");
                     }
                     0
                 }
                 Err(e) => {
                     if self.verbosity.level != VerbosityLevel::Silent {
-                        show_error!("{}", e);
+                        show_error!("{e}");
                     }
                     1
                 }
@@ -350,26 +417,18 @@ impl ChownExecutor {
 
     fn obtain_meta<P: AsRef<Path>>(&self, path: P, follow: bool) -> Option<Metadata> {
         let path = path.as_ref();
-        let meta = if follow {
-            path.metadata()
-        } else {
-            path.symlink_metadata()
-        };
-        match meta {
-            Err(e) => {
-                match self.verbosity.level {
-                    VerbosityLevel::Silent => (),
-                    _ => show_error!(
+        get_metadata(path, follow)
+            .inspect_err(|e| {
+                if self.verbosity.level != VerbosityLevel::Silent {
+                    show_error!(
                         "cannot {} {}: {}",
                         if follow { "dereference" } else { "access" },
                         path.quote(),
-                        strip_errno(&e)
-                    ),
+                        strip_errno(e)
+                    );
                 }
-                None
-            }
-            Ok(meta) => Some(meta),
-        }
+            })
+            .ok()
     }
 
     #[inline]
@@ -381,6 +440,27 @@ impl ChownExecutor {
             IfFrom::UserGroup(u, g) => u == uid && g == gid,
         }
     }
+
+    fn print_verbose_ownership_retained_as(&self, path: &Path, uid: u32, gid: Option<u32>) {
+        if self.verbosity.level == VerbosityLevel::Verbose {
+            let ownership = match (self.dest_uid, self.dest_gid, gid) {
+                (Some(_), Some(_), Some(gid)) => format!(
+                    "{}:{}",
+                    entries::uid2usr(uid).unwrap_or_else(|_| uid.to_string()),
+                    entries::gid2grp(gid).unwrap_or_else(|_| gid.to_string())
+                ),
+                (None, Some(_), Some(gid)) => {
+                    entries::gid2grp(gid).unwrap_or_else(|_| gid.to_string())
+                }
+                _ => entries::uid2usr(uid).unwrap_or_else(|_| uid.to_string()),
+            };
+            if self.verbosity.groups_only {
+                println!("group of {} retained as {ownership}", path.quote());
+            } else {
+                println!("ownership of {} retained as {ownership}", path.quote());
+            }
+        }
+    }
 }
 
 pub mod options {
@@ -412,7 +492,54 @@ pub mod options {
     pub const ARG_FILES: &str = "FILE";
 }
 
-type GidUidFilterParser = fn(&ArgMatches) -> UResult<(Option<u32>, Option<u32>, IfFrom)>;
+pub struct GidUidOwnerFilter {
+    pub dest_gid: Option<u32>,
+    pub dest_uid: Option<u32>,
+    pub raw_owner: String,
+    pub filter: IfFrom,
+}
+type GidUidFilterOwnerParser = fn(&ArgMatches) -> UResult<GidUidOwnerFilter>;
+
+/// Determines symbolic link traversal and recursion settings based on flags.
+/// Returns the updated `dereference` and `traverse_symlinks` values.
+pub fn configure_symlink_and_recursion(
+    matches: &ArgMatches,
+    default_traverse_symlinks: TraverseSymlinks,
+) -> Result<(bool, bool, TraverseSymlinks), Box<dyn crate::error::UError>> {
+    let mut dereference = if matches.get_flag(options::dereference::DEREFERENCE) {
+        Some(true) // Follow symlinks
+    } else if matches.get_flag(options::dereference::NO_DEREFERENCE) {
+        Some(false) // Do not follow symlinks
+    } else {
+        None // Default behavior
+    };
+
+    let mut traverse_symlinks = default_traverse_symlinks;
+    if matches.get_flag("L") {
+        traverse_symlinks = TraverseSymlinks::All
+    } else if matches.get_flag("H") {
+        traverse_symlinks = TraverseSymlinks::First
+    } else if matches.get_flag("P") {
+        traverse_symlinks = TraverseSymlinks::None
+    };
+
+    let recursive = matches.get_flag(options::RECURSIVE);
+    if recursive {
+        if traverse_symlinks == TraverseSymlinks::None {
+            if dereference == Some(true) {
+                return Err(USimpleError::new(
+                    1,
+                    "-R --dereference requires -H or -L".to_string(),
+                ));
+            }
+            dereference = Some(false);
+        }
+    } else {
+        traverse_symlinks = TraverseSymlinks::None;
+    }
+
+    Ok((recursive, dereference.unwrap_or(true), traverse_symlinks))
+}
 
 /// Base implementation for `chgrp` and `chown`.
 ///
@@ -421,11 +548,12 @@ type GidUidFilterParser = fn(&ArgMatches) -> UResult<(Option<u32>, Option<u32>,
 /// `parse_gid_uid_and_filter` will be called to obtain the target gid and uid, and the filter,
 /// from `ArgMatches`.
 /// `groups_only` determines whether verbose output will only mention the group.
+#[allow(clippy::cognitive_complexity)]
 pub fn chown_base(
     mut command: Command,
     args: impl crate::Args,
     add_arg_if_not_reference: &'static str,
-    parse_gid_uid_and_filter: GidUidFilterParser,
+    parse_gid_uid_and_filter: GidUidFilterOwnerParser,
     groups_only: bool,
 ) -> UResult<()> {
     let args: Vec<_> = args.collect();
@@ -468,34 +596,8 @@ pub fn chown_base(
         .unwrap_or_default();
 
     let preserve_root = matches.get_flag(options::preserve_root::PRESERVE);
-
-    let mut dereference = if matches.get_flag(options::dereference::DEREFERENCE) {
-        Some(true)
-    } else if matches.get_flag(options::dereference::NO_DEREFERENCE) {
-        Some(false)
-    } else {
-        None
-    };
-
-    let mut traverse_symlinks = if matches.get_flag(options::traverse::TRAVERSE) {
-        TraverseSymlinks::First
-    } else if matches.get_flag(options::traverse::EVERY) {
-        TraverseSymlinks::All
-    } else {
-        TraverseSymlinks::None
-    };
-
-    let recursive = matches.get_flag(options::RECURSIVE);
-    if recursive {
-        if traverse_symlinks == TraverseSymlinks::None {
-            if dereference == Some(true) {
-                return Err(USimpleError::new(1, "-R --dereference requires -H or -L"));
-            }
-            dereference = Some(false);
-        }
-    } else {
-        traverse_symlinks = TraverseSymlinks::None;
-    }
+    let (recursive, dereference, traverse_symlinks) =
+        configure_symlink_and_recursion(&matches, TraverseSymlinks::None)?;
 
     let verbosity_level = if matches.get_flag(options::verbosity::CHANGES) {
         VerbosityLevel::Changes
@@ -508,21 +610,133 @@ pub fn chown_base(
     } else {
         VerbosityLevel::Normal
     };
-    let (dest_gid, dest_uid, filter) = parse_gid_uid_and_filter(&matches)?;
+    let GidUidOwnerFilter {
+        dest_gid,
+        dest_uid,
+        raw_owner,
+        filter,
+    } = parse_gid_uid_and_filter(&matches)?;
 
     let executor = ChownExecutor {
         traverse_symlinks,
         dest_gid,
         dest_uid,
+        raw_owner,
         verbosity: Verbosity {
             groups_only,
             level: verbosity_level,
         },
         recursive,
-        dereference: dereference.unwrap_or(true),
+        dereference,
         preserve_root,
         files,
         filter,
     };
     executor.exec()
 }
+
+pub fn common_args() -> Vec<Arg> {
+    vec![
+        Arg::new(traverse::TRAVERSE)
+            .short(traverse::TRAVERSE.chars().next().unwrap())
+            .help("if a command line argument is a symbolic link to a directory, traverse it")
+            .overrides_with_all([traverse::EVERY, traverse::NO_TRAVERSE])
+            .action(clap::ArgAction::SetTrue),
+        Arg::new(traverse::EVERY)
+            .short(traverse::EVERY.chars().next().unwrap())
+            .help("traverse every symbolic link to a directory encountered")
+            .overrides_with_all([traverse::TRAVERSE, traverse::NO_TRAVERSE])
+            .action(clap::ArgAction::SetTrue),
+        Arg::new(traverse::NO_TRAVERSE)
+            .short(traverse::NO_TRAVERSE.chars().next().unwrap())
+            .help("do not traverse any symbolic links (default)")
+            .overrides_with_all([traverse::TRAVERSE, traverse::EVERY])
+            .action(clap::ArgAction::SetTrue),
+        Arg::new(options::dereference::DEREFERENCE)
+            .long(options::dereference::DEREFERENCE)
+            .help(
+                "affect the referent of each symbolic link (this is the default), \
+    rather than the symbolic link itself",
+            )
+            .action(clap::ArgAction::SetTrue),
+        Arg::new(options::dereference::NO_DEREFERENCE)
+            .short('h')
+            .long(options::dereference::NO_DEREFERENCE)
+            .help(
+                "affect symbolic links instead of any referenced file \
+        (useful only on systems that can change the ownership of a symlink)",
+            )
+            .action(clap::ArgAction::SetTrue),
+    ]
+}
+
+#[cfg(test)]
+mod tests {
+    // Note this useful idiom: importing names from outer (for mod tests) scope.
+    use super::*;
+    #[cfg(unix)]
+    use std::os::unix;
+    use std::path::{Component, PathBuf};
+    #[cfg(unix)]
+    use tempfile::tempdir;
+
+    #[test]
+    fn test_empty_string() {
+        let path = PathBuf::new();
+        assert_eq!(path.to_str(), Some(""));
+        // The main point to test here is that we don't crash.
+        // The result should be 'false', to avoid unnecessary and confusing warnings.
+        assert!(!is_root(&path, false));
+        assert!(!is_root(&path, true));
+    }
+
+    #[allow(clippy::needless_borrow)]
+    #[cfg(unix)]
+    #[test]
+    fn test_literal_root() {
+        let component = Component::RootDir;
+        let path: &Path = component.as_ref();
+        assert_eq!(
+            path.to_str(),
+            Some("/"),
+            "cfg(unix) but using non-unix path delimiters?!"
+        );
+        // Must return true, this is the main scenario that --preserve-root shall prevent.
+        assert!(is_root(&path, false));
+        assert!(is_root(&path, true));
+    }
+
+    #[cfg(unix)]
+    #[test]
+    fn test_symlink_slash() {
+        let temp_dir = tempdir().unwrap();
+        let symlink_path = temp_dir.path().join("symlink");
+        unix::fs::symlink(PathBuf::from("/"), symlink_path).unwrap();
+        let symlink_path_slash = temp_dir.path().join("symlink/");
+        // Must return true, we're about to "accidentally" recurse on "/",
+        // since "symlink/" always counts as an already-entered directory
+        // Output from GNU:
+        //   $ chown --preserve-root -RH --dereference $(id -u) slink-to-root/
+        //   chown: it is dangerous to operate recursively on 'slink-to-root/' (same as '/')
+        //   chown: use --no-preserve-root to override this failsafe
+        //   [$? = 1]
+        //   $ chown --preserve-root -RH --no-dereference $(id -u) slink-to-root/
+        //   chown: it is dangerous to operate recursively on 'slink-to-root/' (same as '/')
+        //   chown: use --no-preserve-root to override this failsafe
+        //   [$? = 1]
+        assert!(is_root(&symlink_path_slash, false));
+        assert!(is_root(&symlink_path_slash, true));
+    }
+
+    #[cfg(unix)]
+    #[test]
+    fn test_symlink_no_slash() {
+        // This covers both the commandline-argument case and the recursion case.
+        let temp_dir = tempdir().unwrap();
+        let symlink_path = temp_dir.path().join("symlink");
+        unix::fs::symlink(PathBuf::from("/"), &symlink_path).unwrap();
+        // Only return true  we're about to "accidentally" recurse on "/".
+        assert!(!is_root(&symlink_path, false));
+        assert!(is_root(&symlink_path, true));
+    }
+}
