@@ -6,9 +6,10 @@
 
 pub mod display;
 pub mod error;
+#[cfg(feature = "fs")]
+pub mod io;
 pub mod line_ending;
+pub mod locale;
 pub mod os;
 pub mod panic;
-pub mod version_cmp;
-// dir and vdir also need access to the quoting_style module
-pub mod quoting_style;
+pub mod posix;
