@@ -2,9 +2,12 @@
 //
 // For the full copyright and license information, please view the LICENSE
 // file that was distributed with this source code.
-// spell-checker:ignore (words) asdf algo algos asha mgmt xffname hexa GFYEQ HYQK Yqxb
+// spell-checker:ignore (words) asdf algo algos asha mgmt xffname hexa GFYEQ HYQK Yqxb dont
 
-use crate::common::util::TestScenario;
+use uutests::at_and_ucmd;
+use uutests::new_ucmd;
+use uutests::util::TestScenario;
+use uutests::util_name;
 
 const ALGOS: [&str; 11] = [
     "sysv", "bsd", "crc", "md5", "sha1", "sha224", "sha256", "sha384", "sha512", "blake2b", "sm3",
@@ -12,7 +15,7 @@ const ALGOS: [&str; 11] = [
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -75,7 +78,7 @@ fn test_nonexisting_file() {
 
     new_ucmd!()
         .arg(file_name)
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
         .stderr_contains(format!("cksum: {file_name}: No such file or directory"));
 }
@@ -301,24 +304,28 @@ fn test_check_algo() {
         .arg("lorem_ipsum.txt")
         .fails()
         .no_stdout()
-        .stderr_contains("cksum: --check is not supported with --algorithm={bsd,sysv,crc}")
-        .code_is(1);
+        .stderr_contains("cksum: --check is not supported with --algorithm={bsd,sysv,crc,crc32b}");
     new_ucmd!()
         .arg("-a=sysv")
         .arg("--check")
         .arg("lorem_ipsum.txt")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
-        .stderr_contains("cksum: --check is not supported with --algorithm={bsd,sysv,crc}")
-        .code_is(1);
+        .stderr_contains("cksum: --check is not supported with --algorithm={bsd,sysv,crc,crc32b}");
     new_ucmd!()
         .arg("-a=crc")
         .arg("--check")
         .arg("lorem_ipsum.txt")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
-        .stderr_contains("cksum: --check is not supported with --algorithm={bsd,sysv,crc}")
-        .code_is(1);
+        .stderr_contains("cksum: --check is not supported with --algorithm={bsd,sysv,crc,crc32b}");
+    new_ucmd!()
+        .arg("-a=crc32b")
+        .arg("--check")
+        .arg("lorem_ipsum.txt")
+        .fails_with_code(1)
+        .no_stdout()
+        .stderr_contains("cksum: --check is not supported with --algorithm={bsd,sysv,crc,crc32b}");
 }
 
 #[test]
@@ -327,20 +334,18 @@ fn test_length_with_wrong_algorithm() {
         .arg("--length=16")
         .arg("--algorithm=md5")
         .arg("lorem_ipsum.txt")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
-        .stderr_contains("cksum: --length is only supported with --algorithm=blake2b")
-        .code_is(1);
+        .stderr_contains("cksum: --length is only supported with --algorithm=blake2b");
 
     new_ucmd!()
         .arg("--length=16")
         .arg("--algorithm=md5")
         .arg("-c")
         .arg("foo.sums")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
-        .stderr_contains("cksum: --length is only supported with --algorithm=blake2b")
-        .code_is(1);
+        .stderr_contains("cksum: --length is only supported with --algorithm=blake2b");
 }
 
 #[test]
@@ -348,10 +353,9 @@ fn test_length_not_supported() {
     new_ucmd!()
         .arg("--length=15")
         .arg("lorem_ipsum.txt")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
-        .stderr_contains("--length is only supported with --algorithm=blake2b")
-        .code_is(1);
+        .stderr_contains("--length is only supported with --algorithm=blake2b");
 
     new_ucmd!()
         .arg("-l")
@@ -360,10 +364,9 @@ fn test_length_not_supported() {
         .arg("-a")
         .arg("crc")
         .arg("/tmp/xxx")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
-        .stderr_contains("--length is only supported with --algorithm=blake2b")
-        .code_is(1);
+        .stderr_contains("--length is only supported with --algorithm=blake2b");
 }
 
 #[test]
@@ -386,7 +389,7 @@ fn test_length_greater_than_512() {
         .arg("--algorithm=blake2b")
         .arg("lorem_ipsum.txt")
         .arg("alice_in_wonderland.txt")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
         .stderr_is_fixture("length_larger_than_512.expected");
 }
@@ -435,10 +438,9 @@ fn test_raw_multiple_files() {
         .arg("--raw")
         .arg("lorem_ipsum.txt")
         .arg("alice_in_wonderland.txt")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
-        .stderr_contains("cksum: the --raw option is not supported with multiple files")
-        .code_is(1);
+        .stderr_contains("cksum: the --raw option is not supported with multiple files");
 }
 
 #[test]
@@ -447,7 +449,7 @@ fn test_base64_raw_conflicts() {
         .arg("--base64")
         .arg("--raw")
         .arg("lorem_ipsum.txt")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
         .stderr_contains("--base64")
         .stderr_contains("cannot be used with")
@@ -594,7 +596,6 @@ fn test_reset_binary() {
         .stdout_contains("d41d8cd98f00b204e9800998ecf8427e  ");
 }
 
-#[ignore = "issue #6375"]
 #[test]
 fn test_reset_binary_but_set() {
     let scene = TestScenario::new(util_name!());
@@ -611,7 +612,7 @@ fn test_reset_binary_but_set() {
         .arg("--algorithm=md5")
         .arg(at.subdir.join("f"))
         .succeeds()
-        .stdout_contains("d41d8cd98f00b204e9800998ecf8427e *"); // currently, asterisk=false. It should be true
+        .stdout_contains("d41d8cd98f00b204e9800998ecf8427e *");
 }
 
 #[test]
@@ -742,12 +743,11 @@ fn test_conflicting_options() {
         .arg("--binary")
         .arg("--check")
         .arg("f")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
         .stderr_contains(
             "cksum: the --binary and --text options are meaningless when verifying checksums",
-        )
-        .code_is(1);
+        );
 
     scene
         .ucmd()
@@ -755,12 +755,11 @@ fn test_conflicting_options() {
         .arg("-c")
         .arg("-a")
         .arg("md5")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
         .stderr_contains(
             "cksum: the --binary and --text options are meaningless when verifying checksums",
-        )
-        .code_is(1);
+        );
 }
 
 #[test]
@@ -777,10 +776,9 @@ fn test_check_algo_err() {
         .arg("sm3")
         .arg("--check")
         .arg("f")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
-        .stderr_contains("cksum: f: no properly formatted checksum lines found")
-        .code_is(1);
+        .stderr_contains("cksum: f: no properly formatted checksum lines found");
 }
 
 #[test]
@@ -796,10 +794,9 @@ fn test_check_pipe() {
         .arg("--check")
         .arg("-")
         .pipe_in("f")
-        .fails()
+        .fails_with_code(1)
         .no_stdout()
-        .stderr_contains("cksum: 'standard input': no properly formatted checksum lines found")
-        .code_is(1);
+        .stderr_contains("cksum: 'standard input': no properly formatted checksum lines found");
 }
 
 #[test]
@@ -972,29 +969,41 @@ fn test_cksum_check_failed() {
         .arg("CHECKSUM")
         .fails();
 
-    assert!(result
-        .stderr_str()
-        .contains("input: No such file or directory"));
-    assert!(result
-        .stderr_str()
-        .contains("2 lines are improperly formatted\n"));
-    assert!(result
-        .stderr_str()
-        .contains("1 listed file could not be read\n"));
+    assert!(
+        result
+            .stderr_str()
+            .contains("input: No such file or directory")
+    );
+    assert!(
+        result
+            .stderr_str()
+            .contains("2 lines are improperly formatted\n")
+    );
+    assert!(
+        result
+            .stderr_str()
+            .contains("1 listed file could not be read\n")
+    );
     assert!(result.stdout_str().contains("f: OK\n"));
 
     // without strict
     let result = scene.ucmd().arg("--check").arg("CHECKSUM").fails();
 
-    assert!(result
-        .stderr_str()
-        .contains("input: No such file or directory"));
-    assert!(result
-        .stderr_str()
-        .contains("2 lines are improperly formatted\n"));
-    assert!(result
-        .stderr_str()
-        .contains("1 listed file could not be read\n"));
+    assert!(
+        result
+            .stderr_str()
+            .contains("input: No such file or directory")
+    );
+    assert!(
+        result
+            .stderr_str()
+            .contains("2 lines are improperly formatted\n")
+    );
+    assert!(
+        result
+            .stderr_str()
+            .contains("1 listed file could not be read\n")
+    );
     assert!(result.stdout_str().contains("f: OK\n"));
 
     // tests with two files
@@ -1016,15 +1025,21 @@ fn test_cksum_check_failed() {
         .fails();
     println!("result.stderr_str() {}", result.stderr_str());
     println!("result.stdout_str() {}", result.stdout_str());
-    assert!(result
-        .stderr_str()
-        .contains("input2: No such file or directory"));
-    assert!(result
-        .stderr_str()
-        .contains("4 lines are improperly formatted\n"));
-    assert!(result
-        .stderr_str()
-        .contains("2 listed files could not be read\n"));
+    assert!(
+        result
+            .stderr_str()
+            .contains("input2: No such file or directory")
+    );
+    assert!(
+        result
+            .stderr_str()
+            .contains("4 lines are improperly formatted\n")
+    );
+    assert!(
+        result
+            .stderr_str()
+            .contains("2 listed files could not be read\n")
+    );
     assert!(result.stdout_str().contains("f: OK\n"));
     assert!(result.stdout_str().contains("2: OK\n"));
 }
@@ -1094,9 +1109,11 @@ fn test_cksum_mixed() {
     println!("result.stderr_str() {}", result.stderr_str());
     println!("result.stdout_str() {}", result.stdout_str());
     assert!(result.stdout_str().contains("f: OK"));
-    assert!(result
-        .stderr_str()
-        .contains("3 lines are improperly formatted"));
+    assert!(
+        result
+            .stderr_str()
+            .contains("3 lines are improperly formatted")
+    );
 }
 
 #[test]
@@ -1129,7 +1146,6 @@ fn test_cksum_garbage() {
         .stderr_contains("check-file: no properly formatted checksum lines found");
 }
 
-#[ignore = "Should fail on bits"]
 #[test]
 fn test_md5_bits() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -1180,7 +1196,6 @@ fn test_bsd_case() {
         .stderr_contains("f: no properly formatted checksum lines found");
 }
 
-#[ignore = "Different output"]
 #[test]
 fn test_blake2d_tested_with_sha1() {
     let (at, mut ucmd) = at_and_ucmd!();
@@ -1229,9 +1244,7 @@ fn test_check_directory_error() {
 
 #[test]
 fn test_check_base64_hashes() {
-    let hashes =
-        "MD5 (empty) = 1B2M2Y8AsgTpgAmY7PhCfg==\nSHA256 (empty) = 47DEQpj8HBSa+/TImW+5JCeuQeRkm5NMpJWZG3hSuFU=\nBLAKE2b (empty) = eGoC90IBWQPGxv2FJVLScpEvR0DhWEdhiobiF/cfVBnSXhAxr+5YUxOJZESTTrBLkDpoWxRIt1XVb3Aa/pvizg==\n"
-    ;
+    let hashes = "MD5 (empty) = 1B2M2Y8AsgTpgAmY7PhCfg==\nSHA256 (empty) = 47DEQpj8HBSa+/TImW+5JCeuQeRkm5NMpJWZG3hSuFU=\nBLAKE2b (empty) = eGoC90IBWQPGxv2FJVLScpEvR0DhWEdhiobiF/cfVBnSXhAxr+5YUxOJZESTTrBLkDpoWxRIt1XVb3Aa/pvizg==\n";
 
     let scene = TestScenario::new(util_name!());
     let at = &scene.fixtures;
@@ -1276,6 +1289,18 @@ fn test_several_files_error_mgmt() {
         .stderr_contains("incorrect: no properly ");
 }
 
+#[test]
+fn test_check_unknown_checksum_file() {
+    let scene = TestScenario::new(util_name!());
+
+    scene
+        .ucmd()
+        .arg("--check")
+        .arg("missing")
+        .fails()
+        .stderr_only("cksum: missing: No such file or directory\n");
+}
+
 #[test]
 fn test_check_comment_line() {
     // A comment in a checksum file shall be discarded unnoticed.
@@ -1659,12 +1684,13 @@ fn test_check_incorrectly_formatted_checksum_keeps_processing_hex() {
 /// This module reimplements the cksum-base64.pl GNU test.
 mod gnu_cksum_base64 {
     use super::*;
-    use crate::common::util::log_info;
+    use uutests::util::log_info;
 
-    const PAIRS: [(&str, &str); 11] = [
+    const PAIRS: [(&str, &str); 12] = [
         ("sysv", "0 0 f"),
         ("bsd", "00000     0 f"),
         ("crc", "4294967295 0 f"),
+        ("crc32b", "0 0 f"),
         ("md5", "1B2M2Y8AsgTpgAmY7PhCfg=="),
         ("sha1", "2jmj7l5rSw0yVb/vlWAYkK/YBwk="),
         ("sha224", "0UoCjCo6K8lHYQK7KII0xBWisB+CjqYqxbPkLw=="),
@@ -1675,11 +1701,11 @@ mod gnu_cksum_base64 {
         ),
         (
             "sha512",
-            "z4PhNX7vuL3xVChQ1m2AB9Yg5AULVxXcg/SpIdNs6c5H0NE8XYXysP+DGNKHfuwvY7kxvUdBeoGlODJ6+SfaPg=="
+            "z4PhNX7vuL3xVChQ1m2AB9Yg5AULVxXcg/SpIdNs6c5H0NE8XYXysP+DGNKHfuwvY7kxvUdBeoGlODJ6+SfaPg==",
         ),
         (
             "blake2b",
-            "eGoC90IBWQPGxv2FJVLScpEvR0DhWEdhiobiF/cfVBnSXhAxr+5YUxOJZESTTrBLkDpoWxRIt1XVb3Aa/pvizg=="
+            "eGoC90IBWQPGxv2FJVLScpEvR0DhWEdhiobiF/cfVBnSXhAxr+5YUxOJZESTTrBLkDpoWxRIt1XVb3Aa/pvizg==",
         ),
         ("sm3", "GrIdg1XPoX+OYRlIMegajyK+yMco/vt0ftA161CCqis="),
     ];
@@ -1693,10 +1719,10 @@ mod gnu_cksum_base64 {
     }
 
     fn output_format(algo: &str, digest: &str) -> String {
-        if ["sysv", "bsd", "crc"].contains(&algo) {
+        if ["sysv", "bsd", "crc", "crc32b"].contains(&algo) {
             digest.to_string()
         } else {
-            format!("{} (f) = {}", algo.to_uppercase(), digest).replace("BLAKE2B", "BLAKE2b")
+            format!("{} (f) = {digest}", algo.to_uppercase()).replace("BLAKE2B", "BLAKE2b")
         }
     }
 
@@ -1706,6 +1732,7 @@ mod gnu_cksum_base64 {
         let scene = make_scene();
 
         for (algo, digest) in PAIRS {
+            log_info("ALGORITHM", algo);
             scene
                 .ucmd()
                 .arg("--base64")
@@ -1724,8 +1751,17 @@ mod gnu_cksum_base64 {
         let scene = make_scene();
 
         for (algo, digest) in PAIRS {
-            if ["sysv", "bsd", "crc"].contains(&algo) {
+            if ["sysv", "bsd", "crc", "crc32b"].contains(&algo) {
                 // These algorithms do not accept `--check`
+                scene
+                    .ucmd()
+                    .arg("--check")
+                    .arg("-a")
+                    .arg(algo)
+                    .fails()
+                    .stderr_only(
+                        "cksum: --check is not supported with --algorithm={bsd,sysv,crc,crc32b}\n",
+                    );
                 continue;
             }
 
@@ -1792,6 +1828,373 @@ mod gnu_cksum_base64 {
     }
 }
 
+/// This module reimplements the cksum-c.sh GNU test.
+mod gnu_cksum_c {
+    use super::*;
+
+    const INVALID_SUM: &str = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaafdb57c725157cb40b5aee8d937b8351477e";
+
+    fn make_scene() -> TestScenario {
+        let scene = TestScenario::new(util_name!());
+        let at = &scene.fixtures;
+        at.write("input", "9\n7\n1\n4\n2\n6\n3\n5\n8\n10\n");
+
+        let algos: &[&[&str]] = &[
+            &["-a", "sha384"],
+            &["-a", "blake2b"],
+            &["-a", "blake2b", "-l", "384"],
+            &["-a", "sm3"],
+        ];
+
+        for args in algos {
+            let result = scene.ucmd().args(args).succeeds();
+            let stdout = result.stdout();
+            at.append_bytes("CHECKSUMS", stdout);
+        }
+
+        scene
+    }
+
+    #[test]
+    #[ignore]
+    fn test_signed_checksums() {
+        todo!()
+    }
+
+    #[test]
+    fn test_check_individual_digests_in_mixed_file() {
+        let scene = make_scene();
+
+        scene
+            .ucmd()
+            .arg("--check")
+            .arg("-a")
+            .arg("sm3")
+            .arg("CHECKSUMS")
+            .succeeds();
+    }
+
+    #[test]
+    fn test_check_against_older_non_hex_formats() {
+        let scene = make_scene();
+
+        scene
+            .ucmd()
+            .arg("-c")
+            .arg("-a")
+            .arg("crc")
+            .arg("CHECKSUMS")
+            .fails();
+
+        let crc_cmd = scene.ucmd().arg("-a").arg("crc").arg("input").succeeds();
+        let crc_cmd_out = crc_cmd.stdout();
+        scene.fixtures.write_bytes("CHECKSUMS.crc", crc_cmd_out);
+
+        scene.ucmd().arg("-c").arg("CHECKSUMS.crc").fails();
+    }
+
+    #[test]
+    fn test_status() {
+        let scene = make_scene();
+
+        scene
+            .ucmd()
+            .arg("--status")
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .succeeds()
+            .no_output();
+    }
+
+    fn make_scene_with_comment() -> TestScenario {
+        let scene = make_scene();
+
+        scene
+            .fixtures
+            .append("CHECKSUMS", "# Very important comment\n");
+
+        scene
+    }
+
+    #[test]
+    fn test_status_with_comment() {
+        let scene = make_scene_with_comment();
+
+        scene
+            .ucmd()
+            .arg("--status")
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .succeeds()
+            .no_output();
+    }
+
+    fn make_scene_with_invalid_line() -> TestScenario {
+        let scene = make_scene_with_comment();
+
+        scene.fixtures.append("CHECKSUMS", "invalid_line\n");
+
+        scene
+    }
+
+    #[test]
+    fn test_check_strict() {
+        let scene = make_scene_with_invalid_line();
+
+        // without strict, succeeds
+        scene
+            .ucmd()
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .succeeds()
+            .stderr_contains("1 line is improperly formatted");
+
+        // with strict, fails
+        scene
+            .ucmd()
+            .arg("--strict")
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .fails()
+            .stderr_contains("1 line is improperly formatted");
+    }
+
+    fn make_scene_with_two_invalid_lines() -> TestScenario {
+        let scene = make_scene_with_comment();
+
+        scene
+            .fixtures
+            .append("CHECKSUMS", "invalid_line\ninvalid_line\n");
+
+        scene
+    }
+
+    #[test]
+    fn test_check_strict_plural_checks() {
+        let scene = make_scene_with_two_invalid_lines();
+
+        scene
+            .ucmd()
+            .arg("--strict")
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .fails()
+            .stderr_contains("2 lines are improperly formatted");
+    }
+
+    fn make_scene_with_incorrect_checksum() -> TestScenario {
+        let scene = make_scene_with_two_invalid_lines();
+
+        scene
+            .fixtures
+            .append("CHECKSUMS", &format!("SM3 (input) = {INVALID_SUM}\n"));
+
+        scene
+    }
+
+    #[test]
+    fn test_check_with_incorrect_checksum() {
+        let scene = make_scene_with_incorrect_checksum();
+
+        scene
+            .ucmd()
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .fails()
+            .stdout_contains("input: FAILED")
+            .stderr_contains("1 computed checksum did NOT match");
+
+        // also fails with strict
+        scene
+            .ucmd()
+            .arg("--strict")
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .fails()
+            .stdout_contains("input: FAILED")
+            .stderr_contains("1 computed checksum did NOT match");
+    }
+
+    #[test]
+    fn test_status_with_errors() {
+        let scene = make_scene_with_incorrect_checksum();
+
+        scene
+            .ucmd()
+            .arg("--status")
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .fails()
+            .no_output();
+    }
+
+    #[test]
+    fn test_check_with_non_existing_file() {
+        let scene = make_scene();
+        scene
+            .fixtures
+            .write("CHECKSUMS2", &format!("SM3 (input2) = {INVALID_SUM}\n"));
+
+        scene
+            .ucmd()
+            .arg("--check")
+            .arg("CHECKSUMS2")
+            .fails()
+            .stdout_contains("input2: FAILED open or read")
+            .stderr_contains("1 listed file could not be read");
+
+        // also fails with strict
+        scene
+            .ucmd()
+            .arg("--strict")
+            .arg("--check")
+            .arg("CHECKSUMS2")
+            .fails()
+            .stdout_contains("input2: FAILED open or read")
+            .stderr_contains("1 listed file could not be read");
+    }
+
+    fn make_scene_with_another_improperly_formatted() -> TestScenario {
+        let scene = make_scene_with_incorrect_checksum();
+
+        scene.fixtures.append(
+            "CHECKSUMS",
+            &format!("BLAKE2b (missing-file) = {INVALID_SUM}\n"),
+        );
+
+        scene
+    }
+
+    #[test]
+    fn test_warn() {
+        let scene = make_scene_with_another_improperly_formatted();
+
+        scene
+            .ucmd()
+            .arg("--warn")
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .fails()
+            .stderr_contains("CHECKSUMS: 6: improperly formatted SM3 checksum line")
+            .stderr_contains("CHECKSUMS: 9: improperly formatted BLAKE2b checksum line");
+    }
+
+    fn make_scene_with_checksum_missing() -> TestScenario {
+        let scene = make_scene_with_another_improperly_formatted();
+
+        scene.fixtures.write(
+            "CHECKSUMS-missing",
+            &format!("SM3 (nonexistent) = {INVALID_SUM}\n"),
+        );
+
+        scene
+    }
+
+    #[test]
+    fn test_ignore_missing() {
+        let scene = make_scene_with_checksum_missing();
+
+        scene
+            .ucmd()
+            .arg("--ignore-missing")
+            .arg("--check")
+            .arg("CHECKSUMS-missing")
+            .fails()
+            .stdout_does_not_contain("nonexistent: No such file or directory")
+            .stdout_does_not_contain("nonexistent: FAILED open or read")
+            .stderr_contains("CHECKSUMS-missing: no file was verified");
+    }
+
+    #[test]
+    fn test_status_and_warn() {
+        let scene = make_scene_with_checksum_missing();
+
+        // --status before --warn
+        scene
+            .ucmd()
+            .arg("--status")
+            .arg("--warn")
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .fails()
+            .stderr_contains("CHECKSUMS: 9: improperly formatted BLAKE2b checksum line")
+            .stderr_contains("WARNING: 3 lines are improperly formatted")
+            .stderr_contains("WARNING: 1 computed checksum did NOT match");
+
+        // --warn before --status (status hides the results)
+        scene
+            .ucmd()
+            .arg("--warn")
+            .arg("--status")
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .fails()
+            .stderr_does_not_contain("CHECKSUMS: 9: improperly formatted BLAKE2b checksum line")
+            .stderr_does_not_contain("WARNING: 3 lines are improperly formatted")
+            .stderr_does_not_contain("WARNING: 1 computed checksum did NOT match");
+    }
+
+    #[test]
+    fn test_status_and_ignore_missing() {
+        let scene = make_scene_with_checksum_missing();
+
+        scene
+            .ucmd()
+            .arg("--status")
+            .arg("--ignore-missing")
+            .arg("--check")
+            .arg("CHECKSUMS")
+            .fails()
+            .no_output();
+    }
+
+    #[test]
+    fn test_status_warn_and_ignore_missing() {
+        let scene = make_scene_with_checksum_missing();
+
+        scene
+            .ucmd()
+            .arg("--status")
+            .arg("--warn")
+            .arg("--ignore-missing")
+            .arg("--check")
+            .arg("CHECKSUMS-missing")
+            .fails()
+            .stderr_contains("CHECKSUMS-missing: no file was verified")
+            .stdout_does_not_contain("nonexistent: No such file or directory");
+    }
+
+    #[test]
+    fn test_check_several_files_dont_exist() {
+        let scene = make_scene();
+
+        scene
+            .ucmd()
+            .arg("--check")
+            .arg("non-existing-1")
+            .arg("non-existing-2")
+            .fails()
+            .stderr_contains("non-existing-1: No such file or directory")
+            .stderr_contains("non-existing-2: No such file or directory");
+    }
+
+    #[test]
+    fn test_check_several_files_empty() {
+        let scene = make_scene();
+        scene.fixtures.touch("empty-1");
+        scene.fixtures.touch("empty-2");
+
+        scene
+            .ucmd()
+            .arg("--check")
+            .arg("empty-1")
+            .arg("empty-2")
+            .fails()
+            .stderr_contains("empty-1: no properly formatted checksum lines found")
+            .stderr_contains("empty-2: no properly formatted checksum lines found");
+    }
+}
+
 /// The tests in this module check the behavior of cksum when given different
 /// checksum formats and algorithms in the same file, while specifying an
 /// algorithm on CLI or not.
