@@ -1,9 +1,15 @@
-use crate::common::util::{is_ci, TestScenario};
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 use std::env;
+use uutests::new_ucmd;
+use uutests::util::{TestScenario, is_ci};
+use uutests::util_name;
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
