@@ -1,14 +1,19 @@
+// This file is part of the uutils coreutils package.
+//
+// For the full copyright and license information, please view the LICENSE
+// file that was distributed with this source code.
 use std::fs::File;
 
-use crate::common::util::TestScenario;
+use uutests::new_ucmd;
+use uutests::util::TestScenario;
+use uutests::util_name;
 
 #[test]
 #[cfg(not(windows))]
 fn test_dev_null() {
     new_ucmd!()
         .set_stdin(File::open("/dev/null").unwrap())
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stdout_is("not a tty\n");
 }
 
@@ -18,8 +23,7 @@ fn test_dev_null_silent() {
     new_ucmd!()
         .args(&["-s"])
         .set_stdin(File::open("/dev/null").unwrap())
-        .fails()
-        .code_is(1)
+        .fails_with_code(1)
         .stdout_is("");
 }
 
@@ -53,7 +57,7 @@ fn test_close_stdin_silent_alias() {
 
 #[test]
 fn test_wrong_argument() {
-    new_ucmd!().args(&["a"]).fails().code_is(2);
+    new_ucmd!().args(&["a"]).fails_with_code(2);
 }
 
 #[test]
