@@ -4,8 +4,11 @@
 // file that was distributed with this source code.
 
 // spell-checker:ignore nabcd badoption schar
-use crate::common::util::TestScenario;
 use uucore::posix::OBSOLETE;
+use uutests::at_and_ucmd;
+use uutests::new_ucmd;
+use uutests::util::TestScenario;
+use uutests::util_name;
 
 static INPUT: &str = "sorted.txt";
 static OUTPUT: &str = "sorted-output.txt";
@@ -15,7 +18,7 @@ static SORTED_ZERO_TERMINATED: &str = "sorted-zero-terminated.txt";
 
 #[test]
 fn test_invalid_arg() {
-    new_ucmd!().arg("--definitely-invalid").fails().code_is(1);
+    new_ucmd!().arg("--definitely-invalid").fails_with_code(1);
 }
 
 #[test]
@@ -36,7 +39,7 @@ fn test_help_and_version_on_stdout() {
 fn test_stdin_default() {
     new_ucmd!()
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-simple.expected");
 }
 
@@ -44,7 +47,7 @@ fn test_stdin_default() {
 fn test_single_default() {
     new_ucmd!()
         .arg(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-simple.expected");
 }
 
@@ -52,7 +55,7 @@ fn test_single_default() {
 fn test_single_default_output() {
     let (at, mut ucmd) = at_and_ucmd!();
     let expected = at.read("sorted-simple.expected");
-    ucmd.args(&[INPUT, OUTPUT]).run();
+    ucmd.args(&[INPUT, OUTPUT]).succeeds();
     let found = at.read(OUTPUT);
     assert_eq!(found, expected);
 }
@@ -62,7 +65,7 @@ fn test_stdin_counts() {
     new_ucmd!()
         .args(&["-c"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-counts.expected");
 }
 
@@ -71,7 +74,7 @@ fn test_stdin_skip_1_char() {
     new_ucmd!()
         .args(&["-s1"])
         .pipe_in_fixture(SKIP_CHARS)
-        .run()
+        .succeeds()
         .stdout_is_fixture("skip-1-char.expected");
 }
 
@@ -80,7 +83,7 @@ fn test_stdin_skip_5_chars() {
     new_ucmd!()
         .args(&["-s5"])
         .pipe_in_fixture(SKIP_CHARS)
-        .run()
+        .succeeds()
         .stdout_is_fixture("skip-5-chars.expected");
 }
 
@@ -89,7 +92,7 @@ fn test_stdin_skip_and_check_2_chars() {
     new_ucmd!()
         .args(&["-s3", "-w2"])
         .pipe_in_fixture(SKIP_CHARS)
-        .run()
+        .succeeds()
         .stdout_is_fixture("skip-3-check-2-chars.expected");
 }
 
@@ -98,7 +101,7 @@ fn test_stdin_skip_2_fields() {
     new_ucmd!()
         .args(&["-f2"])
         .pipe_in_fixture(SKIP_FIELDS)
-        .run()
+        .succeeds()
         .stdout_is_fixture("skip-2-fields.expected");
 }
 
@@ -107,7 +110,7 @@ fn test_stdin_skip_2_fields_obsolete() {
     new_ucmd!()
         .args(&["-2"])
         .pipe_in_fixture(SKIP_FIELDS)
-        .run()
+        .succeeds()
         .stdout_is_fixture("skip-2-fields.expected");
 }
 
@@ -116,7 +119,7 @@ fn test_stdin_skip_21_fields() {
     new_ucmd!()
         .args(&["-f21"])
         .pipe_in_fixture(SKIP_FIELDS)
-        .run()
+        .succeeds()
         .stdout_is_fixture("skip-21-fields.expected");
 }
 
@@ -125,7 +128,7 @@ fn test_stdin_skip_21_fields_obsolete() {
     new_ucmd!()
         .args(&["-21"])
         .pipe_in_fixture(SKIP_FIELDS)
-        .run()
+        .succeeds()
         .stdout_is_fixture("skip-21-fields.expected");
 }
 
@@ -133,8 +136,7 @@ fn test_stdin_skip_21_fields_obsolete() {
 fn test_stdin_skip_invalid_fields_obsolete() {
     new_ucmd!()
         .args(&["-5q"])
-        .run()
-        .failure()
+        .fails()
         .stderr_contains("error: unexpected argument '-q' found\n");
 }
 
@@ -143,22 +145,22 @@ fn test_stdin_all_repeated() {
     new_ucmd!()
         .args(&["--all-repeated"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-all-repeated.expected");
     new_ucmd!()
         .args(&["--all-repeated=none"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-all-repeated.expected");
     new_ucmd!()
         .args(&["--all-repeated=non"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-all-repeated.expected");
     new_ucmd!()
         .args(&["--all-repeated=n"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-all-repeated.expected");
 }
 
@@ -170,8 +172,7 @@ fn test_all_repeated_followed_by_filename() {
     at.write(filename, "a\na\n");
 
     ucmd.args(&["--all-repeated", filename])
-        .run()
-        .success()
+        .succeeds()
         .stdout_is("a\na\n");
 }
 
@@ -180,17 +181,17 @@ fn test_stdin_all_repeated_separate() {
     new_ucmd!()
         .args(&["--all-repeated=separate"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-all-repeated-separate.expected");
     new_ucmd!()
         .args(&["--all-repeated=separat"]) // spell-checker:disable-line
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-all-repeated-separate.expected");
     new_ucmd!()
         .args(&["--all-repeated=s"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-all-repeated-separate.expected");
 }
 
@@ -199,17 +200,17 @@ fn test_stdin_all_repeated_prepend() {
     new_ucmd!()
         .args(&["--all-repeated=prepend"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-all-repeated-prepend.expected");
     new_ucmd!()
         .args(&["--all-repeated=prepen"]) // spell-checker:disable-line
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-all-repeated-prepend.expected");
     new_ucmd!()
         .args(&["--all-repeated=p"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-all-repeated-prepend.expected");
 }
 
@@ -218,7 +219,7 @@ fn test_stdin_unique_only() {
     new_ucmd!()
         .args(&["-u"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-unique-only.expected");
 }
 
@@ -227,7 +228,7 @@ fn test_stdin_repeated_only() {
     new_ucmd!()
         .args(&["-d"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-repeated-only.expected");
 }
 
@@ -236,7 +237,7 @@ fn test_stdin_ignore_case() {
     new_ucmd!()
         .args(&["-i"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-ignore-case.expected");
 }
 
@@ -245,7 +246,7 @@ fn test_stdin_zero_terminated() {
     new_ucmd!()
         .args(&["-z"])
         .pipe_in_fixture(SORTED_ZERO_TERMINATED)
-        .run()
+        .succeeds()
         .stdout_is_fixture("sorted-zero-terminated.expected");
 }
 
@@ -254,8 +255,7 @@ fn test_gnu_locale_fr_schar() {
     new_ucmd!()
         .args(&["-f1", "locale-fr-schar.txt"])
         .env("LC_ALL", "C")
-        .run()
-        .success()
+        .succeeds()
         .stdout_is_fixture_bytes("locale-fr-schar.txt");
 }
 
@@ -264,7 +264,7 @@ fn test_group() {
     new_ucmd!()
         .args(&["--group"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("group.expected");
 }
 
@@ -276,8 +276,7 @@ fn test_group_followed_by_filename() {
     at.write(filename, "a\na\n");
 
     ucmd.args(&["--group", filename])
-        .run()
-        .success()
+        .succeeds()
         .stdout_is("a\na\n");
 }
 
@@ -286,12 +285,12 @@ fn test_group_prepend() {
     new_ucmd!()
         .args(&["--group=prepend"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("group-prepend.expected");
     new_ucmd!()
         .args(&["--group=p"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("group-prepend.expected");
 }
 
@@ -300,12 +299,12 @@ fn test_group_append() {
     new_ucmd!()
         .args(&["--group=append"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("group-append.expected");
     new_ucmd!()
         .args(&["--group=a"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("group-append.expected");
 }
 
@@ -314,17 +313,17 @@ fn test_group_both() {
     new_ucmd!()
         .args(&["--group=both"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("group-both.expected");
     new_ucmd!()
         .args(&["--group=bot"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("group-both.expected");
     new_ucmd!()
         .args(&["--group=b"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("group-both.expected");
 }
 
@@ -333,18 +332,18 @@ fn test_group_separate() {
     new_ucmd!()
         .args(&["--group=separate"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("group.expected");
     new_ucmd!()
         .args(&["--group=s"])
         .pipe_in_fixture(INPUT)
-        .run()
+        .succeeds()
         .stdout_is_fixture("group.expected");
 }
 
 #[test]
 fn test_case2() {
-    new_ucmd!().pipe_in("a\na\n").run().stdout_is("a\n");
+    new_ucmd!().pipe_in("a\na\n").succeeds().stdout_is("a\n");
 }
 
 struct TestCase {
@@ -1179,6 +1178,17 @@ fn test_stdin_w1_multibyte() {
     new_ucmd!()
         .args(&["-w1"])
         .pipe_in(input)
-        .run()
+        .succeeds()
         .stdout_is("à\ná\n");
 }
+
+#[cfg(target_os = "linux")]
+#[test]
+fn test_failed_write_is_reported() {
+    new_ucmd!()
+        .pipe_in("hello")
+        .args(&["-z"])
+        .set_stdout(std::fs::File::create("/dev/full").unwrap())
+        .fails()
+        .stderr_is("uniq: write error: No space left on device\n");
+}
